/* 
 *  $Id: dia.c,v 1.5 2003/11/26 15:54:14 kogan Exp $
 *
 *  Copyright (c) 2003, Sergey Kogan
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#include <PalmOS.h>
#include "config.h"
#include "dia.h"
#include "resource.h"
#include "mmcd.h"
#include "kwp2/kwp2.h"
#include "vb12/vb12.h"
#include "mp72/mp7.h"
#include "mp72/mp72.h"
#include "mp73/mp73.h"
#include "gmvz/gmvz.h"

DiaHandle diaModList[DIA_NMODS] = {

	/* gmvz, VAZ-GM-ISFI-2S, resources=10000 */
	{
		"VAZ-GM",
		'gmvz',
		64,	/* sample data size */
		4,
		(DiaSensorType*)gmvzSensorDefault,
		0,
		&gmvzTimingDefault,
		&gmvzTiming,
		0,
		(DiaCtlType*)gmvzctl,

		GmvzStart,
		GmvzStop,
		GmvzRun,
		GmvzIsRunning,
		GmvzIsConnected,
		GmvzIsPolling,
		GmvzGetLastError,
		Kwp2SetMask,
		GmvzStartSensorsPoll,
		GmvzStopSensorsPoll,
		GmvzGetSample,
		GmvzCtl,
		GmvzCtlGetResult,
		GmvzPrefsDialog,
		GmvzAboutDialog,
		GmvzLoadPrefs,
		GmvzSavePrefs
	},

	/* kwp2, VAZ-M1.5.4-Russia83, resources=11000 */
	{
		"VAZ-M1.5.4",
		'kwp2',
		40,	/* sample data size */
		33,
		(DiaSensorType*)kwp2SensorDefault,
		0,
		&kwp2TimingDefault,
		&kwp2Timing,
		0,
		(DiaCtlType*)kwp2ctl,

		Kwp2Start,
		Kwp2Stop,
		Kwp2Run,
		Kwp2IsRunning,
		Kwp2IsConnected,
		Kwp2IsPolling,
		Kwp2GetLastError,
		Kwp2SetMask,
		Kwp2StartSensorsPoll,
		Kwp2StopSensorsPoll,
		Kwp2GetSample,
		Kwp2Ctl,
		Kwp2CtlGetResult,
		Kwp2PrefsDialog,
		Kwp2AboutDialog,
		Kwp2LoadPrefs,
		Kwp2SavePrefs
	},

	/* vb12, VAZ-M1.5.4-Euro2, resources=11200 */
	{
		"VAZ-M1.5.4N",
		'vb12',
		40,	/* sample data size */
		33,
		(DiaSensorType*)vb12SensorDefault,
		0,
		&vb12TimingDefault,
		&vb12Timing,
		0,
		(DiaCtlType*)kwp2ctl,

		Kwp2Start,
		Kwp2Stop,
		Kwp2Run,
		Kwp2IsRunning,
		Kwp2IsConnected,
		Kwp2IsPolling,
		Kwp2GetLastError,
		Kwp2SetMask,
		Kwp2StartSensorsPoll,
		Kwp2StopSensorsPoll,
		Kwp2GetSample,
		Kwp2Ctl,
		Kwp2CtlGetResult,
		Vb12PrefsDialog,
		Vb12AboutDialog,
		Kwp2LoadPrefs,
		Kwp2SavePrefs
	},

	/* mp72, VAZ-MP7.0-Euro2, resources=11300 */
	{
		"VAZ-MP7.0-E2",
		'mp72',
		48,	/* sample data size */
		33,	/* sensor count */
		(DiaSensorType*)mp72SensorDefault,
		0,
		&mp72TimingDefault,
		&mp72Timing,
		0,
		(DiaCtlType*)mp72ctl,

		Mp7Start,
		Mp7Stop,
		Mp7Run,
		Mp7IsRunning,
		Mp7IsConnected,
		Mp7IsPolling,
		Mp7GetLastError,
		Kwp2SetMask,
		Mp7StartSensorsPoll,
		Mp7StopSensorsPoll,
		Mp7GetSample,
		Mp7Ctl,
		Mp7CtlGetResult,
		Mp7PrefsDialog,
		Mp7AboutDialog,
		Mp7LoadPrefs,
		Mp7SavePrefs
	},

	/* mp72, VAZ-MP7.0-Euro3, resources=11400 */
	{
		"VAZ-MP7.0-E3",
		'mp73',
		100,	/* sample data size */
		33,
		(DiaSensorType*)mp73SensorDefault,
		0,
		&mp73TimingDefault,
		&mp73Timing,
		0,
		(DiaCtlType*)mp73ctl,

		Mp7Start,
		Mp7Stop,
		Mp7Run,
		Mp7IsRunning,
		Mp7IsConnected,
		Mp7IsPolling,
		Mp7GetLastError,
		Kwp2SetMask,
		Mp7StartSensorsPoll,
		Mp7StopSensorsPoll,
		Mp7GetSample,
		Mp7Ctl,
		Mp7CtlGetResult,
		Mp7PrefsDialog,
		Mp7AboutDialog,
		Mp7LoadPrefs,
		Mp7SavePrefs
	},
};

DiaHandle *diaMod = &diaModList[0];
UInt8 diaCurrentMod = 0;

//DiaSensorType diaSensor[33];

DIA_DEFINE_FMT(DEC)
{
	return StrPrintF(fmt, "%u", UINT16(*data));
}

DIA_DEFINE_FMT(PERCENT)
{
	Int16 r = f_DEC(selfP, data, fmt);
	StrCat(fmt, "%");
	return r+1;
}

DIA_DEFINE_SCALE(DECBYTE)
{
	return vsize - 1 - (UINT16(*data)*UINT16(vsize))/256;
}

DIA_DEFINE_SCALE(DECWORD)
{
	return vsize - 1 - UINT8(UINT32(*((UInt16*)data))*UINT32(vsize)/65536L);
}

DIA_DEFINE_COMPARE(BYTE)
{
	if(*dataA > *dataB) return 1;
	if(*dataA < *dataB) return -1;
	return 0;
}

DIA_DEFINE_COMPARE(WORD)
{
	if(*((UInt16*)dataA) > *((UInt16*)dataB)) return 1;
	if(*((UInt16*)dataA) < *((UInt16*)dataB)) return -1;
	return 0;
}

/*
// Help Dialog
#define HelpForm							10400
#define HelpField							10402
#define HelpDoneButton						10403
#define HelpUpButton						10404
#define HelpDownButton						10405

static void DiaHelpScroll (FormType * formP, WinDirectionType direction)
{
	UInt16 upIndex;
	UInt16 downIndex;
	UInt16 linesToScroll;
	Boolean enableUp;
	Boolean enableDown;
	FieldType * fldP;

	fldP = FrmGetObjectPtr (formP, FrmGetObjectIndex (formP, HelpField));
	linesToScroll = FldGetVisibleLines (fldP) - 1;
	FldScrollField (fldP, linesToScroll, direction);

	enableUp = FldScrollable (fldP, winUp);
	enableDown = FldScrollable (fldP, winDown);
	upIndex = FrmGetObjectIndex (formP, HelpUpButton);
	downIndex = FrmGetObjectIndex (formP, HelpDownButton);

	FrmUpdateScrollers (formP, upIndex, downIndex, enableUp, enableDown);
}

void DiaSensorsHelp()
{
	UInt16 upIndex;
	UInt16 downIndex;
	FormType * formP;
	FieldType * fldP;
	Boolean enableUp;
	Boolean enableDown;
	Boolean done = false;
	EventType event;
	FormActiveStateType	curFrmState;
	Char text[1024];
	Char title[32];
	UInt8 i;

	text[0] = 0;
	for(i = 0; i < 32; i++)
		if(diaMod->sensorDefault[i].exists)
		{
			StrCat(text, "\225 ");
			StrCat(text, diaMod->sensorDefault[i].slug);
			StrCat(text, "\t- ");
			StrCat(text, diaMod->sensorDefault[i].desc);
			StrCat(text, "\n");
		}

	// Command Bar should always be cleared before opening a dialog
	MenuEraseStatus(0);

	FrmSaveActiveState(&curFrmState);		// save active form/window state

	InsPtEnable (false);

	formP = FrmInitForm (HelpForm);

	fldP = FrmGetObjectPtr (formP, FrmGetObjectIndex (formP, HelpField));
	FldSetTextPtr (fldP, text);
	FldRecalculateField (fldP, true);

	StrCopy(title, diaMod->slug);
	StrCat(title, " sensors");
	FrmSetTitle(formP, title);

	FrmSetActiveForm (formP);
	FrmDrawForm (formP);
	
	upIndex = FrmGetObjectIndex (formP, HelpUpButton);
	downIndex = FrmGetObjectIndex (formP, HelpDownButton);

	enableUp = FldScrollable (fldP, winUp);
	enableDown = FldScrollable (fldP, winDown);
	FrmUpdateScrollers (formP, upIndex, downIndex, enableUp, enableDown);
	
	while (! done)
		{
		EvtGetEvent (&event, evtWaitForever);
		if (! SysHandleEvent (&event))
			FrmHandleEvent (formP, &event);

		if (event.eType == ctlSelectEvent)
			{
			if (event.data.ctlSelect.controlID == HelpDoneButton)
				done = true;
			}
					
		else if (event.eType == ctlRepeatEvent)
			{
			switch (event.data.ctlRepeat.controlID)
				{
				case HelpUpButton:
					DiaHelpScroll (formP, winUp);
					break;
					
				case HelpDownButton:
					DiaHelpScroll (formP, winDown);
					break;
				}
			}


		else if	(	(event.eType == keyDownEvent)
	 				&&	(!TxtCharIsHardKey(	event.data.keyDown.modifiers,
													event.data.keyDown.chr))
					&&	(EvtKeydownIsVirtual(&event)))
			{
			switch (event.data.keyDown.chr)
				{
				case vchrPageUp:
					DiaHelpScroll (formP, winUp);
					break;
					
				case vchrPageDown:
					DiaHelpScroll (formP, winDown);
					break;
				}
			}


		else if (event.eType == appStopEvent)
			{
			EvtAddEventToQueue (&event);
			break;
			}
		}

	FldSetTextHandle (fldP, 0);
	FrmEraseForm (formP);
	FrmDeleteForm (formP);
	
	FrmRestoreActiveState(&curFrmState);		// restore active form/window state

}
*/

UInt32 _diaTimingReplyCounter = 0;
UInt32 _diaTimingErrorCounter = 0;
Boolean _diaTimingRunning = false;

void *_DiaTimingGetObjPtrFromActiveForm(UInt16 objectID)
{
	return FrmGetObjectPtr(FrmGetActiveForm(),
		FrmGetObjectIndex(FrmGetActiveForm(), objectID));
}

void _DiaTimingSetLabel(UInt16 objID, Char *text)
{
	FormPtr frmP = FrmGetActiveForm();
	UInt16 idx = FrmGetObjectIndex(frmP, objID);
	FrmHideObject(frmP, idx);
	if(StrLen(text) > 26)
		text[26] = 0;
	FrmCopyLabel(frmP, objID, text);	// 26 max
	FrmShowObject(frmP, idx);
}

void _DiaTimingShow()
{
	Char buf[16];

	StrPrintF(buf, "%lu", diaMod->timing->maxP1);
	_DiaTimingSetLabel(timingValueP1Id, buf);
	CtlSetValue((ControlType *)_DiaTimingGetObjPtrFromActiveForm(timingSliderP1Id), (Int16)diaMod->timing->maxP1/10);
	StrPrintF(buf, "%lu", diaMod->timing->maxP2);
	_DiaTimingSetLabel(timingValueP2Id, buf);
	CtlSetValue((ControlType *)_DiaTimingGetObjPtrFromActiveForm(timingSliderP2Id), (Int16)diaMod->timing->maxP2/10);
	StrPrintF(buf, "%lu", diaMod->timing->minP3);
	_DiaTimingSetLabel(timingValueP3Id, buf);
	CtlSetValue((ControlType *)_DiaTimingGetObjPtrFromActiveForm(timingSliderP3Id), (Int16)diaMod->timing->minP3/10);
}

Boolean _DiaTimingDialogHandleEvent(EventPtr eventP)
{
	DiaSampleType *sampleP = NULL;
	Char *lastErrorP = NULL;
	Char buf[128];
	static UInt8 string2sensor[32];
	static UInt8 _diaTimingSensor;

	switch (eventP->eType) 
	{
	case frmOpenEvent:
		{
			UInt16 i;
		  	ListPtr list;
		  	FormPtr frmP = FrmGetActiveForm();
			static Char *strings[32];
			UInt8 strings_size = 0;

	        _DiaTimingShow();

		  	for(i = 0; i < 32; i++)
		  		if(diaMod->sensor[i].exists)
		  		{
			  		strings[strings_size] = diaMod->sensor[i].slug;
			  		string2sensor[strings_size] = i;
                    strings_size++;
                }

			list = FrmGetObjectPtrById(frmP, timingPopupListId);
			LstSetListChoices(list, strings, strings_size);
			LstSetHeight(list, strings_size < 10 ? strings_size : 10);
			LstSetSelection(list, 0);
            _diaTimingSensor = string2sensor[0];
			CtlSetLabel(FrmGetObjectPtrById(frmP, timingTriggerId),
				diaMod->sensor[_diaTimingSensor].slug
			);
		}
		return true;

	case popSelectEvent:
		switch(eventP->data.popSelect.controlID)
		{
		case timingTriggerId:
			if(_diaTimingSensor == string2sensor[eventP->data.popSelect.selection])
				return true;

            _diaTimingSensor = string2sensor[eventP->data.popSelect.selection];
			CtlSetLabel(FrmGetObjectPtrById(FrmGetActiveForm(), timingTriggerId),
				diaMod->sensor[_diaTimingSensor].slug
			);
			return true;
		}
		break;

	case ctlSelectEvent:
		switch (eventP->data.ctlSelect.controlID)
		{
		case timingStartId:
			if(diaMod->DiaIsRunning()) return true;

			{
				FormPtr frmP = FrmGetActiveForm();
				UInt16 idx = FrmGetObjectIndex(frmP, okButtonId);
				FrmHideObject(frmP, idx);
			}
//            _DiaTimingSetP();
   			_DiaTimingSetLabel(timingResultId, "");
   			_DiaTimingSetLabel(timingErrorId, "");

			_diaTimingReplyCounter = 0;
			_diaTimingErrorCounter = 0;

			diaMod->DiaStart(sensorCaptureMask);
			_diaTimingRunning = true;
			diaMod->DiaStartSensorsPoll();
			
			{
			   	EventType event;
				do {
					EvtGetEvent(&event, 0);
					FrmDispatchEvent(&event);
				} while(_diaTimingRunning);
			}

			return true;

		case timingStopId:
			if(!diaMod->DiaIsRunning()) return true;
			diaMod->DiaStopSensorsPoll();
			_diaTimingRunning = false;
			diaMod->DiaStop();
   			_DiaTimingSetLabel(timingResultId, "stopped");
   			_DiaTimingSetLabel(timingErrorId, "---");
			{
				FormPtr frmP = FrmGetActiveForm();
				UInt16 idx = FrmGetObjectIndex(frmP, okButtonId);
				FrmShowObject(frmP, idx);
			}
			return true;

		case timingDefaultId:
			*diaMod->timing = *diaMod->timingDefault;
        	_DiaTimingShow();
	        return true;
		}
	    break;

	case ctlRepeatEvent:
		switch(eventP->data.ctlRepeat.controlID) {
		case timingSliderP1Id:
			{
				Char buf[16];

				diaMod->timing->maxP1 = eventP->data.ctlRepeat.value*10;
				StrPrintF(buf, "%lu", diaMod->timing->maxP1);
				_DiaTimingSetLabel(timingValueP1Id, buf);
			}
			return false;
		case timingSliderP2Id:
			{
				Char buf[16];

				diaMod->timing->maxP2 = eventP->data.ctlRepeat.value*10;
				StrPrintF(buf, "%lu", diaMod->timing->maxP2);
				_DiaTimingSetLabel(timingValueP2Id, buf);
			}
			return false;
		case timingSliderP3Id:
			{
				Char buf[16];

				diaMod->timing->minP3 = eventP->data.ctlRepeat.value*10;
				StrPrintF(buf, "%lu", diaMod->timing->minP3);
				_DiaTimingSetLabel(timingValueP3Id, buf);
			}
			return false;
		}
	    break;
	default:
	}  

    switch(diaMod->DiaRun())
    {
    case DIA_RESULT:		// maybe got sample
    	sampleP = diaMod->DiaGetSample();
    	if(sampleP != NULL)
   		{
			// show it
			static Char buf[24];

			diaMod->sensor[_diaTimingSensor].format(
					&diaMod->sensor[_diaTimingSensor],
					&sampleP->data[diaMod->sensor[_diaTimingSensor].addr],
					diaMod->sensor[_diaTimingSensor].value);

			StrPrintF(buf, "%lu %s", _diaTimingReplyCounter++, diaMod->sensor[_diaTimingSensor].value);
			_DiaTimingSetLabel(timingResultId, buf);
   		}
    	break;
    case DIA_ERROR:
   		lastErrorP = diaMod->DiaGetLastError();
		StrPrintF(buf, "%lu ", _diaTimingErrorCounter++);
    	StrCat(buf, lastErrorP);
		_DiaTimingSetLabel(timingErrorId, buf);
    	break;
    default:
    }


	return false;
}

void DiaTimingDialog()
{
	if(diaMod->DiaIsRunning()) return;

	{
		EventType event;
		Char title[32];
		FormType *formP = FrmInitForm(timingDialogId);

		FrmSetEventHandler(formP, _DiaTimingDialogHandleEvent);

		StrCopy(title, diaMod->slug);
		StrCat(title, " timing");
		FrmSetTitle(formP, title);

		MemSet (&event, sizeof(EventType), 0);

		event.eType = frmOpenEvent;
		event.data.frmOpen.formID = timingDialogId;
		EvtAddEventToQueue (&event);

		FrmDoDialog(formP);
		FrmDeleteForm(formP);
	}
}

