/* 
 *  $Id: dia.h,v 1.5 2003/11/26 15:54:14 kogan Exp $
 *
 *  Copyright (c) 2003, Sergey Kogan
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#ifndef DIA_H
#define DIA_H

/* typecast macros */
#define INT8(a)		((Int8)(a))
#define INT16(a)	((Int16)(a))
#define INT32(a)	((Int32)(a))
#define UINT8(a)	((UInt8)(a))
#define UINT16(a)	((UInt16)(a))
#define UINT32(a)	((UInt32)(a))

/* types */

typedef struct _DiaTimingType {
	UInt32				minP1;
	UInt32				maxP1;
	UInt32				minP2;
	UInt32				maxP2;
	UInt32				minP3;
	UInt32				maxP3;
	UInt32				minP4;
	UInt32				maxP4;
} DiaTimingType;

typedef struct _DiaPrefsType {
	UInt8				reserved[64];
} DiaPrefsType;

typedef struct _DiaSampleType {
	UInt32				time;			/* in seconds */
	UInt32				sensorPresent;	/* one bit for each sensor, not byte */
	UInt8				data[128];
} DiaSampleType;

typedef enum _DiaReturnType {
	DIA_OK,
	DIA_ERROR,
	DIA_RESULT
} DiaReturnType;

struct _DiaSensorType;

/*
DiaFormatType

Int16 DiaFormatType(DiaSensorType *selfP, UInt8 *data, Char *fmt);
*/
typedef Int16 (*DiaFormatType)(struct _DiaSensorType*, UInt8*, Char*);
#define DIA_DEFINE_FMT(a) Int16 f_##a(struct _DiaSensorType *selfP, UInt8 *data, Char *fmt)

/*
DiaScaleType - sensor graph scale function

UInt8 DiaScaleType(DiaSensorType *selfP, UInt8 *data, UInt8 vsize)

Return value: y coordinate of point, 0..(vsize-1)
*/
typedef UInt8 (*DiaScaleType)(struct _DiaSensorType*, UInt8*, UInt8);
#define DIA_DEFINE_SCALE(a) UInt8 s_##a(struct _DiaSensorType *selfP, UInt8 *data, UInt8 vsize)

/*
DiaCompareType - compares sensor value with threshold

Int8 DiaCompareType(DiaSensorType *selfP, UInt8 *dataA, UInt8 *dataB)
*/
typedef Int8 (*DiaCompareType)(struct _DiaSensorType*, UInt8*, UInt8*);
#define DIA_DEFINE_COMPARE(a) Int8 c_##a(struct _DiaSensorType *selfP, UInt8 *dataA, UInt8 *dataB)


typedef struct _DiaSensorType {
	UInt8 			addr;				/* sensor offset in sample			*/
	UInt32			mask;				/* sensor capture mask				*/
	Char			slug[5];			/* sensor acronym					*/

	DiaFormatType	format;				/* pointer to formatting func		*/
	DiaScaleType	scale;				/* pointer to scaling func			*/
	DiaCompareType	compare;			/* pointer to treshold compare func	*/

	UInt8			exists   : 1;		/* sensor exists					*/

	UInt8			sound	 : 4;		/* not used							*/
	UInt8			polarity : 1;       /* not used							*/
	UInt16			threshold;          /* not used							*/
	RGBColorType	color;				/* not used							*/

	Char			value[12];			/* formatted sensor value			*/
	const Char*		desc;				/* sensor description				*/
} DiaSensorType;


/*
 *  Ctl handlers
 */

/* ctl types */
#define DIA_CTL_RESERVED_0		0
#define DIA_CTL_ECU_RESET		1
#define DIA_CTL_READ_IDENT		2
#define DIA_CTL_CLEAR_ERRORS	3
#define DIA_CTL_READ_ERRORS		4
#define DIA_CTL_USER			5

typedef enum {
	DIA_SEND,
	DIA_RECV,
} DiaCtlOp;

typedef DiaReturnType (*DiaCtlType)(UInt8 ctl, DiaCtlOp op, void *param);

#define DIA_DEFINE_CTL(a)	\
DiaReturnType ctl_##a(UInt8 ctl, DiaCtlOp op, void *param)



/* standard format functions */
DIA_DEFINE_FMT(DEC);
DIA_DEFINE_FMT(PERCENT);

/* standard scale functions */
DIA_DEFINE_SCALE(DECBYTE);
DIA_DEFINE_SCALE(DECWORD);

/* standard compare functions */
DIA_DEFINE_COMPARE(BYTE);
DIA_DEFINE_COMPARE(WORD);

void DiaTimingDialog();
//void DiaSensorsHelp();

/* protocol handler */

typedef struct _DiaHandle
{

	/* parameters */

	Char			slug[16];	/* module slug			*/
	UInt32			id;		/* four character protocol ID	*/
	UInt8			sampleDataSize;	/* actual sample data size	*/
	UInt8			sensorCount;
	DiaSensorType		*sensor;	/* array of sensors		*/
	UInt32			capture;
	const DiaTimingType	*timingDefault;
	DiaTimingType		*timing;
	UInt32			mask;		/* capture mask			*/
	DiaCtlType		*ctl;		/* control handlers table	*/

	/* methods */

	/* start/stop diagnostic session */
	DiaReturnType	(*DiaStart)(UInt32 mask);
	DiaReturnType	(*DiaStop)();

	/* call this function as frequently as possible */
	DiaReturnType	(*DiaRun)();

	/* driver state */
	Boolean			(*DiaIsRunning)();
	Boolean			(*DiaIsConnected)();
	Boolean			(*DiaIsPolling)();
	Char*			(*DiaGetLastError)(); /* return NULL if no errors */

	/* polling */
	DiaReturnType	(*DiaSetMask)(UInt32 mask);
	DiaReturnType	(*DiaStartSensorsPoll)();
	DiaReturnType	(*DiaStopSensorsPoll)();
	DiaSampleType*	(*DiaGetSample)();	/* return NULL if sample is not ready, flush sample */

	/* operations */
	DiaReturnType	(*DiaCtl)(UInt8 ctl, void *param);
	/* return FALSE if result is not ready */
	Boolean			(*DiaCtlGetResult)(UInt8 *ctl, void **param, UInt16 *size);

	void			(*DiaPrefsDialog)();
	void			(*DiaAboutDialog)();

	void			(*DiaLoadPrefs)(DiaPrefsType *prefs);
	void			(*DiaSavePrefs)(DiaPrefsType *prefs);

	void			(*DiaDisableCapture)(UInt8 sensor);
	void			(*DiaEnableCapture)(UInt8 sensor);

} DiaHandle;

#define DIA_NMODS		5

extern DiaHandle	diaModList[DIA_NMODS];
extern DiaHandle*	diaMod;
extern UInt8		diaCurrentMod;

#endif
