/* 
 *  $Id: gmvz.c,v 1.4 2003/11/27 13:58:58 kogan Exp $
 *
 *  Copyright (c) 2003, Sergey Kogan
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#include <PalmOS.h>
#include "../config.h"
#include "../dia.h"
#include "gmvz.h"
#include "format.h"


/* sensors */
const DiaSensorType gmvzSensorDefault[] = {
//
// E - exists, Sn - sound, P - polarity,
// Thr - threshold, Clr - color
//
//addr  mask        slug    format	scale		compare		E  Sn P Thr color						value        desc
{ 0x00, 0x00000000, "XXXX", f_DEC,	s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

{ 0x16, 0x00000001, "RPM",  f_gmvzRPM,	s_gmvzRPM,	c_WORD,		1, 0, 0, 0, {0, 0xFF, 0x00, 0x00},	"?\0      ", "Engine speed" },
{ 0x14, 0x00000001, "TPS",  f_gmvzTPS,	s_gmvzTPS,	c_BYTE,		1, 0, 0, 0, {0, 0xFF, 0x00, 0xFF},	"?\0      ", "Throttle position" },
{ 0x18, 0x00000001, "VS",   f_gmvzSPD,	s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0x00, 0xA0},	"?\0      ", "Vehicle speed" },
};


#define GMVZ_CTL_CONNECT		DIA_CTL_USER+0
#define GMVZ_CTL_POLL			DIA_CTL_USER+1
#define GMVZ_CTL_DISCONNECT		DIA_CTL_USER+2
#define GMVZ_CTL_PING			DIA_CTL_USER+3

DIA_DEFINE_CTL(GmvzEcuReset		) GMVZ_SECT;
DIA_DEFINE_CTL(GmvzReadIdent	) GMVZ_SECT;
DIA_DEFINE_CTL(GmvzClearErrors	) GMVZ_SECT;
DIA_DEFINE_CTL(GmvzReadErrors	) GMVZ_SECT;

DIA_DEFINE_CTL(GmvzConnect		) GMVZ_SECT;
DIA_DEFINE_CTL(GmvzPoll			) GMVZ_SECT;
DIA_DEFINE_CTL(GmvzDisconnect	) GMVZ_SECT;
DIA_DEFINE_CTL(GmvzPing			) GMVZ_SECT;

DiaCtlType gmvzctl[] = {
NULL,
ctl_GmvzEcuReset,		/* ECU_RESET		*/
ctl_GmvzReadIdent,		/* READ_IDENT		*/
ctl_GmvzClearErrors,	/* CLEAR_ERRORS		*/
ctl_GmvzReadErrors,		/* READ_ERRORS		*/

ctl_GmvzConnect,		/* CONNECT			*/
ctl_GmvzPoll,			/* POLL				*/
ctl_GmvzDisconnect,		/* DISCONNECT		*/
ctl_GmvzPing,			/* DISCONNECT		*/
};

typedef enum _GmvzWaitingFor {
	DIA_WAIT_NONE,
	DIA_WAIT_SEND,
	DIA_WAIT_ECHO,
	DIA_WAIT_HDR,
	DIA_WAIT_REPLY
} GmvzWaitingFor;

typedef struct _GmvzRequestType {
	UInt8				ctl;			// parameters passed to GmvzCtl()
	void*				ctl_param;			
	
	UInt8				send[256];
	UInt8				send_size;
	UInt32				send_ticks;

	UInt32				sample;			// sample start point
	UInt32				res0;			// reserved
	UInt8				recv[256];
	UInt8				recv_size;
	UInt8				recv_checksum;
	UInt8				recv_to;

	UInt8				retry;			// retries counter
} GmvzRequestType;

/*** internal vars ***/

Boolean				_gmvzRunning			= false;
Boolean				_gmvzConnected			= false;
Boolean				_gmvzPolling			= false;
Boolean				_gmvzErrorMode			= false;
Boolean				_gmvzError				= false;	// TRUE if last operation was unsuccessful
Char				_gmvzLastError[64];

GmvzWaitingFor		_gmvzWaitingFor			= DIA_WAIT_NONE;
GmvzRequestType		_gmvzReq;

DiaSampleType*		_gmvzSampleP			= NULL;
Boolean				_gmvzSampleReady		= false;

UInt16				_gmvzPortId;
UInt16				_gmvzAutoOff;
Boolean				_gmvzPortOpen			= false;

UInt32				_gmvzTicksPerSecond;
UInt32				_gmvzLastReplyTicks		= 0;

const DiaTimingType	gmvzTimingDefault = {
	0,			/* Min P1 */
	20,			/* Max P1 */
	25,			/* Min P2 */
	60,			/* Max P2 */
	110,		/* Min P3 */
	5000,		/* Max P3 */
	0,			/* Min P4 */
	20			/* Max P4 */
};
DiaTimingType		gmvzTiming;

//UInt32				_gmvzMinP1				= 0;
//UInt32				_gmvzMaxP1				= 20;
//UInt32				_gmvzMinP2				= 25;
//UInt32				_gmvzMaxP2				= 60;
//UInt32				_gmvzMinP3				= 110;
//UInt32				_gmvzMaxP3				= 5000;
//UInt32				_gmvzMinP4				= 0;
//UInt32				_gmvzMaxP4				= 20;

/*** API implementation ***/

#define MSEC2TICKS(a)			(((a)*_gmvzTicksPerSecond)/1000)
//#define DIA_SLEEP_MSEC(a)		SysTaskDelay(((a)*_gmvzTicksPerSecond)/1000)

/* internal functions decls */
DiaReturnType		_GmvzTryToSend() GMVZ_SECT;
DiaReturnType		_GmvzTryToRecv() GMVZ_SECT;
DiaReturnType		_GmvzError(Char *msg, Err err, Boolean fatal) GMVZ_SECT; // always return DIA_ERROR
DiaReturnType		_GmvzOnIdle() GMVZ_SECT;
DiaReturnType		_GmvzClosePort() GMVZ_SECT;
DiaReturnType		_GmvzOnRecv() GMVZ_SECT;
Int16				_GmvzCompare(const UInt8 *s1, const UInt8 *s2, UInt8 length) GMVZ_SECT;
DiaReturnType		_GmvzSendAsync(UInt8 *req, UInt8 size, UInt8 recv_to) GMVZ_SECT;
DiaReturnType		_GmvzOpenPort(UInt32 baudRate, Boolean enableFlowControl) GMVZ_SECT;


inline void	_GmvzClearErrors()
{
	_gmvzError = false;
}

/* start/stop diagnostic session */
DiaReturnType GmvzStart(UInt32 mask)
{
	if(_gmvzRunning) return DIA_OK;

	diaMod->mask = mask;

    _GmvzClearErrors();
	_gmvzRunning = true;
	_gmvzConnected = false;
	_gmvzPolling = false;
	_gmvzWaitingFor = DIA_WAIT_NONE;
	_gmvzPortOpen = false;
	_gmvzTicksPerSecond = SysTicksPerSecond();
	_gmvzSampleReady = false;

	_gmvzAutoOff = SysSetAutoOffTime(0);

	return DIA_OK;
}

DiaReturnType GmvzStop()
{
	if(!_gmvzRunning) return DIA_OK;

	if(_gmvzPolling)
		GmvzStopSensorsPoll();	// stop polling

	// stop current request
	if(_gmvzWaitingFor == DIA_WAIT_NONE || _gmvzWaitingFor == DIA_WAIT_SEND)
		_gmvzWaitingFor = DIA_WAIT_NONE;
	else
		while(1)
			if(GmvzRun() != DIA_OK) break;
	_gmvzSampleReady = false;

	GmvzCtl(GMVZ_CTL_DISCONNECT, NULL);	// close connection synchronously

//	if(_gmvzPortOpen)
//		_GmvzClosePort();

	if(_gmvzAutoOff) {
		SysSetAutoOffTime(_gmvzAutoOff);
		_gmvzAutoOff = 0;
	}

	_gmvzRunning = false;

	return DIA_OK;
}

/* call this function as frequently as possible */
DiaReturnType GmvzRun()
{
	DiaReturnType return_event = DIA_OK;

	if(!_gmvzRunning) return DIA_OK;

	_gmvzSampleReady = false;

	if(_gmvzWaitingFor != DIA_WAIT_NONE)
	{
		if(_gmvzWaitingFor == DIA_WAIT_SEND)
			return_event = _GmvzTryToSend();
		else
			return_event = _GmvzTryToRecv();
	}
	else
	// idle, check what we can do
	if(!_gmvzConnected)
		return_event = GmvzCtl(GMVZ_CTL_CONNECT, NULL);	// try to connect
	else
	if(_gmvzPolling && diaMod->mask != 0)
		return_event = GmvzCtl(GMVZ_CTL_POLL, NULL);
	else
		return_event = _GmvzOnIdle(); // send ping to ECU
	return return_event;
}

/* driver states */
Boolean GmvzIsRunning()
{
	return _gmvzRunning;
}

Boolean GmvzIsConnected()
{
	return _gmvzConnected;
}

Boolean GmvzIsPolling()
{
	return _gmvzPolling;
}

Char* GmvzGetLastError() /* return NULL if no errors */
{
	if(!_gmvzError) return NULL;
	return _gmvzLastError;
}


/* polling */
DiaReturnType GmvzStartSensorsPoll()
{
	if(!_gmvzRunning) return DIA_ERROR;
	_gmvzPolling = true;
	_gmvzSampleReady = false;
	return DIA_OK;
}

DiaReturnType GmvzStopSensorsPoll()
{
	if(!_gmvzRunning) return DIA_ERROR;
	_gmvzPolling = false;
	return DIA_OK;
}

DiaSampleType* GmvzGetSample()	/* return NULL if sample is not ready,
								flush sample */
{
#ifndef EMULATE_ECU_ANSWERS
	if(!_gmvzPolling) return NULL;
	if(!_gmvzSampleReady) return NULL;
	_gmvzSampleReady = false;
#else
	_gmvzSampleP = (DiaSampleType *)&_gmvzReq.sample;
	_gmvzSampleP->time = TimGetSeconds();
	_gmvzSampleP->sensorPresent = 0xFFFFFFFF;
	_gmvzSampleP->data[9]++;
#endif
	return _gmvzSampleP;
}


/* operations */
DiaReturnType GmvzCtl(UInt8 ctl, void *param)
{
	DiaCtlType ctlP = NULL;
    DiaReturnType r;

	if(_gmvzWaitingFor != DIA_WAIT_NONE) return _GmvzError("GmvzCtl(): driver busy", errNone, false);

	_GmvzClearErrors();
	_gmvzSampleReady = false;

	ctlP = diaMod->ctl[ctl];
	if(ctlP == NULL)
		return _GmvzError("null ctl called", errNone, false);

	_gmvzReq.ctl = ctl;
	_gmvzReq.ctl_param = param;

	r = ctlP(ctl, DIA_SEND, param);
	if(r != DIA_OK) return r;
	r =_GmvzTryToSend();
	if(r != DIA_OK) return r;
	r = _GmvzTryToRecv();
	if(r != DIA_OK) return r;

	return DIA_OK;
}

DiaReturnType _GmvzOnRecv()
{
	DiaCtlType ctlP = NULL;

	ctlP = diaMod->ctl[_gmvzReq.ctl];
	return ctlP(_gmvzReq.ctl, DIA_RECV, _gmvzReq.ctl_param);
}

Boolean GmvzCtlGetResult(UInt8 *ctl, void **param,
						UInt16 *size)	/* return FALSE if result
										is not ready */
{
	return _GmvzError("Not implemented", errNone, true);
}


/*** internal functions implementation ***/

DiaReturnType _GmvzTryToSend()
{
    if(_gmvzWaitingFor != DIA_WAIT_SEND) return DIA_OK;

	if(TimGetTicks() - _gmvzLastReplyTicks >= MSEC2TICKS(gmvzTiming.minP3))
	{
		{
			Err err;

			SrmReceiveFlush(_gmvzPortId, 0);
			SrmSend(_gmvzPortId, _gmvzReq.send, _gmvzReq.send_size, &err);
			if(err != errNone)
				return _GmvzError("SrmSend()", err, true);

            _gmvzReq.send_ticks = TimGetTicks();
		}

		_gmvzWaitingFor = DIA_WAIT_ECHO;
	}
	return DIA_OK;
}

Int16 _GmvzCompare(const UInt8 *s1, const UInt8 *s2, UInt8 length)
{
	UInt8 i;
	Int16 r = 0;
	for(i = 0; i < length; i++)
		if((r = *s1++ - *s2++)) return r;
	return r;
}


DiaReturnType _GmvzTryToRecv()
{
	Err err;
	UInt32 bytes;
	UInt32 bytes_expected;
	UInt32 timeout;

	switch(_gmvzWaitingFor)
	{
	case DIA_WAIT_ECHO:
		bytes_expected = _gmvzReq.send_size;
		timeout = MSEC2TICKS(gmvzTiming.maxP2);
		break;
	case DIA_WAIT_HDR:
		bytes_expected = 3;
		timeout = MSEC2TICKS(gmvzTiming.maxP2 + 3*gmvzTiming.maxP1);
		break;
	case DIA_WAIT_REPLY:
		bytes_expected = _gmvzReq.recv_size + 1;
		timeout = MSEC2TICKS(gmvzTiming.maxP2 + (bytes_expected + 2)*gmvzTiming.maxP1);
		break;
	default:
		return DIA_OK;
	}

	err = SrmReceiveCheck(_gmvzPortId, &bytes);
	if(err != errNone)
		return _GmvzError("SrmReceiveCheck()", err, true);

	if(bytes >= bytes_expected)
	{
		switch(_gmvzWaitingFor)
		{
		case DIA_WAIT_ECHO:
			// recv echo
			SrmReceive(_gmvzPortId, _gmvzReq.recv, bytes_expected, 1, &err);
			if(err != errNone)
				return _GmvzError("SrmReceive()", err, true);
			// compare
			if(_GmvzCompare(_gmvzReq.send, _gmvzReq.recv, _gmvzReq.send_size) != 0)
				return _GmvzError("Invalid echo", errNone, true);
			// go to the next step
			_gmvzWaitingFor = DIA_WAIT_HDR;
			return _GmvzTryToRecv();
		case DIA_WAIT_HDR:
			// get packet length
			SrmReceive(_gmvzPortId, _gmvzReq.recv, 3, 1, &err);
			if(err != errNone)
				return _GmvzError("SrmReceive()", err, true);

            _gmvzReq.recv_checksum = _gmvzReq.recv[0];
			_gmvzReq.recv_size = _gmvzReq.recv[1] - 0x56;
			_gmvzReq.recv_checksum += _gmvzReq.recv[1];
			_gmvzReq.recv_checksum += _gmvzReq.recv[2];

			if(_gmvzReq.recv[2] != _gmvzReq.send[2])
				return _GmvzError("Unknown answer", errNone, true);

			// go to the next step
			_gmvzWaitingFor = DIA_WAIT_REPLY;
			return _GmvzTryToRecv();
		case DIA_WAIT_REPLY:
			// read reply
			SrmReceive(_gmvzPortId, _gmvzReq.recv+_gmvzReq.recv_to, _gmvzReq.recv_size+1, 1, &err);
			if(err != errNone)
				return _GmvzError("SrmReceive()", err, true);

			// store reply ticks
			_gmvzLastReplyTicks = TimGetTicks();

			// check checksum
			{
				UInt8 i;
				for(i = 0; i < _gmvzReq.recv_size; i++)
					_gmvzReq.recv_checksum += _gmvzReq.recv[_gmvzReq.recv_to+i];
			}
			_gmvzReq.recv_checksum = (~_gmvzReq.recv_checksum)+1;
			if(_gmvzReq.recv_checksum != _gmvzReq.recv[_gmvzReq.recv_to+_gmvzReq.recv_size])
				return _GmvzError("Bad checksum", errNone, true);

			// clear WaitingFor
			_gmvzWaitingFor = DIA_WAIT_NONE;
			// call OnRecv();
			return _GmvzOnRecv();
		default:
			return DIA_ERROR;
		}
	}
	else
		if(TimGetTicks() - _gmvzReq.send_ticks > timeout)
		{
			if(_gmvzReq.retry >= 10) 
				return _GmvzError("Recv timeout", errNone, false);
			else
			{
				_gmvzReq.retry++;
				if(bytes > 0)
					SrmReceiveFlush(_gmvzPortId, 0);
				_gmvzWaitingFor = DIA_WAIT_SEND;
				return _GmvzTryToSend();
			}
		}

	return DIA_OK;
}

DiaReturnType _GmvzOnIdle()
{
	if(_gmvzWaitingFor == DIA_WAIT_NONE &&
		(TimGetTicks() - _gmvzLastReplyTicks) > MSEC2TICKS(gmvzTiming.maxP3 >> 1))
		return GmvzCtl(GMVZ_CTL_PING, NULL);
	return DIA_OK;
}

DiaReturnType _GmvzError(Char *msg, Err err, Boolean fatal) // always return true
{
	// check error mode
	if(_gmvzErrorMode) return DIA_ERROR;

	// enter error mode
	_gmvzErrorMode = true;

	// reset internal states
	_gmvzWaitingFor = DIA_WAIT_NONE;

	if(fatal)
		GmvzCtl(GMVZ_CTL_DISCONNECT, NULL);

	// set error
	StrCopy(_gmvzLastError, msg);
	if(err != errNone)
	{
		Char buf[64];

		if(SysErrString(err, buf, 64) != NULL)
		{
			StrCat(_gmvzLastError, ": ");
			StrCat(_gmvzLastError, buf);
		}
	}
	_gmvzError = true;

	// flush input/output buffer
	SrmReceiveFlush(_gmvzPortId, 0);
	SrmSendFlush(_gmvzPortId);
	// clear Srm errors
	SrmClearErr(_gmvzPortId);
	// leave error mode
	_gmvzErrorMode = false;
	return DIA_ERROR;//_GmvzError("Not implemented", errNone, true);
}

DiaReturnType _GmvzSendAsync(UInt8 *req, UInt8 size, UInt8 recv_to)
{
	UInt8 checksum = 0;
	UInt8 i;

	if(size > 0xA9)
		return _GmvzError("_GmvzSendAsync(): Can't send long packet", errNone, false);

	checksum += _gmvzReq.send[0] = 0xF4;
	checksum += _gmvzReq.send[1] = 0x56 + size - 1;
	for(i = 0; i < size; i++)
		checksum += _gmvzReq.send[i+2] = req[i];
	_gmvzReq.send[size+2] = (~checksum) + 1;
	_gmvzReq.send_size = size+3;
	_gmvzReq.recv_to = recv_to;

	_gmvzReq.retry = 0;
	_gmvzWaitingFor = DIA_WAIT_SEND;

	return DIA_OK;
}

DiaReturnType _GmvzOpenPort(UInt32 baudRate, Boolean enableFlowControl)
{
	Err err;

	if(_gmvzPortOpen) return DIA_OK;

	err = SrmOpen(serPortCradleRS232Port, baudRate, &_gmvzPortId);
	if(err != errNone)
		err = SrmOpen(serPortCradlePort, baudRate, &_gmvzPortId);
	if(err != errNone)
		return _GmvzError("SrmOpen()", err, true);

	_gmvzPortOpen = true;
		
	if(!enableFlowControl) {
		Int16 paramSize;
		UInt32 flags =	srmSettingsFlagBitsPerChar8 | srmSettingsFlagStopBits1;
		paramSize = sizeof(flags);
		err = SrmControl(_gmvzPortId, srmCtlSetFlags, &flags, &paramSize);
		if(err != errNone)
			return _GmvzError("SrmControl()", err, true);
	}

	return DIA_OK;
}

DiaReturnType _GmvzClosePort()
{
	Err err;

	if(!_gmvzPortOpen) return DIA_OK;

	_gmvzPortOpen = false;
	err = SrmClose(_gmvzPortId);
	if(err != errNone)
		return _GmvzError("SrmClose()", err, false);

	return DIA_OK;
}

/*
 *  ctl handlers
 */

DIA_DEFINE_CTL(GmvzConnect)
{
	if(op == DIA_SEND)
	{
		DiaReturnType r;
		UInt8 req[] = { 0x00 };

		if(_gmvzConnected) return DIA_RESULT;

		r = _GmvzOpenPort(8192, false);
		if(r != DIA_OK) return r;

		return _GmvzSendAsync(req, 1, 0);
	}
	// check result
	if(_gmvzReq.recv_size != 0)
		return _GmvzError("Inv. answer length", errNone, true);

	_gmvzConnected = true;

	return DIA_RESULT;
}

DIA_DEFINE_CTL(GmvzDisconnect)
{
	if(op == DIA_SEND)
	{
		UInt8 req[] = { 0x00 };

		// do disconnect
		if(_gmvzConnected)
		{
			_GmvzSendAsync(req, 1, 0);
		
			while(1)
				if(GmvzRun() != DIA_OK) break;

			_gmvzConnected = false;
		}

		_GmvzClosePort();
	}

	return DIA_RESULT;
}

DIA_DEFINE_CTL(GmvzPoll)
{
	if(op == DIA_SEND)
	{
		UInt8 req[] = { 0x01, 0x00 };
		return _GmvzSendAsync(req, 2, 0);
	}
	_gmvzSampleP = (DiaSampleType *)&_gmvzReq.sample;
	_gmvzSampleP->time = TimGetSeconds();
	_gmvzSampleP->sensorPresent = 0xFFFFFFFF;
	_gmvzSampleReady = true;
	return DIA_RESULT;
}

DIA_DEFINE_CTL(GmvzPing)
{
	if(op == DIA_SEND)
	{
		UInt8 req[] = { 0x01, 0x00 };
		return _GmvzSendAsync(req, 2, 0);
	}
	return DIA_OK;
}

DIA_DEFINE_CTL(GmvzReadIdent)
{
	return _GmvzError("Not implemented", errNone, true);
}

DIA_DEFINE_CTL(GmvzClearErrors)
{
	return _GmvzError("Not implemented", errNone, true);
}

DIA_DEFINE_CTL(GmvzReadErrors)
{
	return _GmvzError("Not implemented", errNone, true);
}

DIA_DEFINE_CTL(GmvzEcuReset)
{
	return _GmvzError("Not implemented", errNone, true);
}
