/* 
 *  $Id: format.c,v 1.4 2003/11/26 15:54:15 kogan Exp $
 *
 *  Copyright (c) 2003, Sergey Kogan
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#include <PalmOS.h>
#include "../config.h"
#include "../dia.h"

#define DEG "\260"	// degree symbol

/*
 *  format functions
 */

// E
DIA_DEFINE_FMT(kwp2VS)
{
	return StrPrintF(fmt, "%ukm/h", UINT16(*data));
}

// E - 40
DIA_DEFINE_FMT(kwp2ECT)
{
	return StrPrintF(fmt, "%d" DEG "C", INT16(*data) - 40);
}

// E/256
DIA_DEFINE_FMT(kwp2COCR)
{
	UInt32 v = UINT32(*data) * 256L;
	return StrPrintF(fmt, "%ld.%03ld", v >> 16, (v & 0xffff) * 1000L >> 16);
}

// E * 40
DIA_DEFINE_FMT(kwp2RPM)
{
	UInt16 rpm = UINT16(*data)*40;
	return StrPrintF(fmt, "%drpm", rpm);
}

// E * 10
DIA_DEFINE_FMT(kwp2XRPM)
{
	return StrPrintF(fmt, "%drpm", UINT16(*data)*10);
}

// 5.2 + E * 0.05
DIA_DEFINE_FMT(kwp2BATT)
{
	UInt32 v = 340787L + 3277L * UINT32(*data);
	return StrPrintF(fmt, "%ld.%ldV", v >> 16, (v & 0xffff) * 10 >> 16);
}

// E / 10
DIA_DEFINE_FMT(kwp2MAF)
{
	UInt16 val = UINT16(*(data+1))<<8|UINT16(*data);
	return StrPrintF(fmt, "%u.%ukg/h", val / 10, val % 10);
}

// E / 6
DIA_DEFINE_FMT(kwp2CAF)
{
	UInt32 v = UINT32(UINT16(*(data+1))<<8|UINT16(*data)) * 10992L;
	return StrPrintF(fmt, "%ld.%01ldmg/c", v >> 16, (v & 0xffff) * 10L >> 16);
}

// E / 2
DIA_DEFINE_FMT(kwp2SA)
{
	Int16 val = INT16(*((Int8*)data));
	return StrPrintF(fmt, "%d.%d" DEG, val/2, abs(5*(val%2)));
}

// (E + 128)/256
DIA_DEFINE_FMT(kwp2IPC)
{
	UInt32 v = (UINT32(*data) + 128L) * 256L;
	return StrPrintF(fmt, "%ld.%03ld", v >> 16, (v & 0xffff) * 1000L >> 16);

}

// E/125
DIA_DEFINE_FMT(kwp2IPW)
{
	UInt32 v = UINT32(UINT16(*(data+1))<<8|UINT16(*data)) * 524L;
	return StrPrintF(fmt, "%ld.%03ldms", v >> 16, (v & 0xffff) * 1000L >> 16);

}

// E/128
DIA_DEFINE_FMT(kwp2FUEL)
{
	UInt32 v = //(UINT32(data[0]) | UINT32(data[1]) << 8);
		UINT32(UINT16(*(data+1))<<8|UINT16(*data));
	if(v == 0xFFFF) {
		StrCopy(fmt, "inf");
		return 3;
	}
	v *= 512L;
	return StrPrintF(fmt, "%ld.%ldl", v >> 16, (v & 0xffff) * 10L >> 16);
}

// E/50
DIA_DEFINE_FMT(kwp2HFUE)
{
	UInt32 v = 
		UINT32(UINT16(*(data+1))<<8|UINT16(*data))
		* 1311L;
	return StrPrintF(fmt, "%ld.%ldl/h", v >> 16, (v & 0xffff) * 10L >> 16);
}

/* 
DIA_DEFINE_FMT(kwp2KNCK)
{
	Char b[5] = "- --";
	if(*(data+1) & 0x20) b[0] = 'x';
	if(*(data+1) & 0x08) b[2] = 'x';
	if(*data & 0x20) b[3] = 'x';
	StrCopy(fmt, b);
	return 4;
}
*/

// E * 5.0 / 256
DIA_DEFINE_FMT(kwp2ADC)
{
	UInt32 v = UINT32(*data) * 1280L;
	return StrPrintF(fmt, "%ld.%02ldV", v >> 16, (v & 0xffff) * 100L >> 16);
}

// E * 5.0 * 0.287 / 256;
DIA_DEFINE_FMT(kwp2BATv)
{
	UInt32 v = UINT32(*data) * 367L;
	return StrPrintF(fmt, "%ld.%02ldV", v >> 16, (v & 0xffff) * 100L >> 16);
}

// (E + 128) * 14.7 / 256;
DIA_DEFINE_FMT(kwp2AFR)
{
	UInt32 v = (UINT32(*data) + 128L) * 3763L;
	return StrPrintF(fmt, "%ld.%02ld", v >> 16, (v & 0xffff) * 100L >> 16);
}

/*
 *  scaling functions
 */

DIA_DEFINE_SCALE(kwp2RPM)
{
	return vsize - 1 - UINT8((UINT16(*data)*vsize)/200);
}

DIA_DEFINE_SCALE(kwp2TPS)
{
	return vsize - 1 - UINT8((UINT16(*data)*vsize)/110);
}

DIA_DEFINE_SCALE(kwp2MAF)
{
	return vsize - 1 - UINT8(
		UINT32(UINT16(*(data+1))<<8|UINT16(*data))
		*UINT32(vsize)/2500L);
}

DIA_DEFINE_SCALE(kwp2CAF)
{
	return vsize - 1 - UINT8(
		UINT32(UINT16(*(data+1))<<8|UINT16(*data))
		*UINT32(vsize)/2800L);
}

DIA_DEFINE_SCALE(kwp2IPW)
{
	return vsize - 1 - UINT8(
		UINT32(UINT16(*(data+1))<<8|UINT16(*data))
		*UINT32(vsize)/2600L);
}

DIA_DEFINE_SCALE(kwp2FUEL)
{
	UInt32 v = UINT32(UINT16(*(data+1))<<8|UINT16(*data));
	if(v == 0xFFFF) return 0;
	return vsize - 1 - UINT8(v*UINT32(vsize)/65536L);
}

DIA_DEFINE_SCALE(kwp2HFUE)
{
	return vsize - 1 - UINT8(
		UINT32(UINT16(*(data+1))<<8|UINT16(*data))
		*UINT32(vsize)/1300L);
}

/*
DIA_DEFINE_SCALE(kwp2KNCK)
{
	UInt8 val = 0;

	if(*(data+1) & 0x20)
		val = vsize - 5;
	else
	if(*(data+1) & 0x08)
		val = vsize - vsize/3;
	else
	if(*data & 0x20)
		val = vsize/3;

	return vsize - 1 - val;
}
*/

DIA_DEFINE_SCALE(kwp2SA)
{
	return vsize - 1 - UINT8(((INT16(*((Int8*)data))+16)*vsize)/112);
}
