/* 
 *  $Id: kwp2.c,v 1.5 2003/11/26 15:54:15 kogan Exp $
 *
 *  Copyright (c) 2003, Sergey Kogan
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#include <PalmOS.h>
#include "../config.h"
#include "../dia.h"
#include "kwp2.h"
#include "format.h"


/* sensors */
const DiaSensorType kwp2SensorDefault[] = {
//
// E - exists, Sn - sound, P - polarity,
// Thr - threshold, Clr - color
//
//addr  mask         slug    format	    	scale		compare		E  Sn P Thr color						value        desc
{ 0x00, 0x00000000L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

{ 0x09, 0x00000001L, "RPM",  f_kwp2RPM,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xFF, 0x00, 0x00},	"?\0      ", "Engine speed" },
{ 0x10, 0x00000001L, "IPW",  f_kwp2IPW,		s_kwp2IPW,	c_WORD,		1, 0, 0, 0, {0, 0xFF, 0xA0, 0x00},	"?\0      ", "Injector pulse width" },
{ 0x08, 0x00000001L, "TPS",  f_PERCENT,		s_kwp2TPS,	c_BYTE,		1, 0, 0, 0, {0, 0xFF, 0x00, 0xFF},	"?\0      ", "Throttle position" },
{ 0x16, 0x00000001L, "HFUE", f_kwp2HFUE,	s_kwp2HFUE,	c_WORD,		1, 0, 0, 0, {0, 0xFF, 0xA0, 0xA0},	"?\0      ", "Hour fuel consumption" },
{ 0x0E, 0x00000001L, "VS",   f_kwp2VS,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0x00, 0xA0},	"?\0      ", "Vehicle speed" },
{ 0x12, 0x00000001L, "MAF",  f_kwp2MAF,		s_kwp2MAF,	c_WORD,		1, 0, 0, 0, {0, 0x00, 0x00, 0xFF},	"?\0      ", "Mass air flow" },
{ 0x0D, 0x00000001L, "SA",   f_kwp2SA,		s_kwp2SA,	c_BYTE,		1, 0, 0, 0, {0, 0x60, 0x60, 0x60},	"?\0      ", "Spark advance" },
{ 0x14, 0x00000001L, "CAF",  f_kwp2CAF,		s_kwp2CAF,	c_WORD,		1, 0, 0, 0, {0, 0xA0, 0xA0, 0xFF},	"?\0      ", "Cycle air flow" },

{ 0x06, 0x00000001L, "ECT",  f_kwp2ECT,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0xFF, 0x00},	"?\0      ", "Engine coolant temperature" },
{ 0x0C, 0x00000001L, "IPC",  f_kwp2IPC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xA0, 0xA0, 0x00},	"?\0      ", "Injector pulse correction" },
{ 0x0F, 0x00000001L, "BATT", f_kwp2BATT,	s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xA0, 0xFF, 0xA0},	"?\0      ", "Battery voltage" },
{ 0x18, 0x00000001L, "FUEL", f_kwp2FUEL,	s_kwp2FUEL,	c_WORD,		1, 0, 0, 0, {0, 0xA0, 0x60, 0x00},	"?\0      ", "Fuel consumption" },
{ 0x0A, 0x00000001L, "XRPM", f_kwp2XRPM,	s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xA0, 0x00, 0xA0},	"?\0      ", "Engine idle speed" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x0B, 0x00000001L, "ISCP", f_DEC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0xFF, 0xFF},	"?\0      ", "Idle speed control position" },
{ 0x07, 0x00000001L, "COCR", f_kwp2COCR,	s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xD0, 0xD0, 0xD0},	"?\0      ", "CO correction" },

{ 0x1e, 0x00000002L, "KNKv", f_kwp2ADC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x30, 0x30, 0x30},	"?\0      ", "Knock sensor voltage" },
{ 0x1f, 0x00000002L, "ECTv", f_kwp2ADC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0xA0, 0x00},	"?\0      ", "ECT sensor voltage" },
{ 0x20, 0x00000002L, "MAFv", f_kwp2ADC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0x00, 0xA0},	"?\0      ", "Mass air flow sensor voltage" },
{ 0x21, 0x00000002L, "BATv", f_kwp2BATv,	s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x60, 0xA0, 0x60},	"?\0      ", "Battery voltage" },
{ 0x22, 0x00000002L, "COv",  f_kwp2ADC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x60, 0x60, 0x60},	"?\0      ", "CO correction voltage" },
{ 0x23, 0x00000002L, "TPSv", f_kwp2ADC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xA0, 0x00, 0xA0},	"?\0      ", "Throttle position sensor voltage" },
//{ 0x04, 0x00000001L, "KNCK", f_kwp2KNCK,	s_kwp2KNCK,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

};

#define KWP2_CTL_CONNECT			DIA_CTL_USER+0
#define KWP2_CTL_POLL				DIA_CTL_USER+1
#define KWP2_CTL_DISCONNECT			DIA_CTL_USER+2
#define KWP2_CTL_PING				DIA_CTL_USER+3
#define KWP2_CTL_RECONNECT			DIA_CTL_USER+4

DIA_DEFINE_CTL(Kwp2EcuReset		) KWP2_SECT;
DIA_DEFINE_CTL(Kwp2ReadIdent	) KWP2_SECT;
DIA_DEFINE_CTL(Kwp2ClearErrors	) KWP2_SECT;
DIA_DEFINE_CTL(Kwp2ReadErrors	) KWP2_SECT;
DIA_DEFINE_CTL(Kwp2Connect		) KWP2_SECT;
DIA_DEFINE_CTL(Kwp2Poll			) KWP2_SECT;
DIA_DEFINE_CTL(Kwp2Disconnect	) KWP2_SECT;
DIA_DEFINE_CTL(Kwp2Ping			) KWP2_SECT;
DIA_DEFINE_CTL(Kwp2Reconnect	) KWP2_SECT;

DiaCtlType kwp2ctl[] = {
NULL,					/* RESERVED_0		*/
ctl_Kwp2EcuReset,		/* ECU_RESET		*/
ctl_Kwp2ReadIdent,		/* READ_IDENT		*/
ctl_Kwp2ClearErrors,	/* CLEAR_ERRORS		*/
ctl_Kwp2ReadErrors,		/* READ_ERRORS		*/

ctl_Kwp2Connect,		/* CONNECT			*/
ctl_Kwp2Poll,			/* POLL				*/
ctl_Kwp2Disconnect,		/* DISCONNECT		*/
ctl_Kwp2Ping,			/* PING				*/
ctl_Kwp2Reconnect,		/* RECONNECT		*/
};

typedef enum _Kwp2WaitingFor {
	DIA_WAIT_NONE,
	DIA_WAIT_SEND,
	DIA_WAIT_ECHO,
	DIA_WAIT_HDR,
	DIA_WAIT_REPLY
} Kwp2WaitingFor;

typedef struct _Kwp2RequestType {
	UInt8				ctl;			/* parameters passed to Kwp2Ctl()	*/
	void*				ctl_param;			
	
	UInt8				send[256];
	UInt8				send_size;
	UInt32				send_ticks;

	UInt32				sample;			/* sample start point				*/
	UInt32				res0;			/* reserved							*/
	UInt8				recv[256];
	UInt8				recv_size;
	UInt8				recv_checksum;
	UInt8				recv_to;		/* recv next reply to recv[recv_to]	*/

	UInt8				retry;			/* retries counter					*/
} Kwp2RequestType;

/*** internal vars ***/

Boolean				_kwp2Running			= false;
Boolean				_kwp2Connected			= false;
Boolean				_kwp2Polling			= false;
Boolean				_kwp2ErrorMode			= false;
Boolean				_kwp2Error				= false;	// TRUE if last operation was unsuccessful
UInt8				_kwp2ErrorCounter		= 0;
Char				_kwp2LastError[64];

Kwp2WaitingFor		_kwp2WaitingFor			= DIA_WAIT_NONE;
Kwp2RequestType		_kwp2Req;

DiaSampleType*		_kwp2SampleP			= NULL;
Boolean				_kwp2SampleReady		= false;

UInt16				_kwp2PortId;
UInt16				_kwp2AutoOff;
Boolean				_kwp2PortOpen			= false;

UInt32				_kwp2TicksPerSecond;
UInt32				_kwp2LastReplyTicks		= 0;

const DiaTimingType	kwp2TimingDefault = {
	0,			/* Min P1 */
	20,			/* Max P1 */
	25,			/* Min P2 */
	60,			/* Max P2 */
	110,		/* Min P3 */
	5000,		/* Max P3 */
	0,			/* Min P4 */
	20			/* Max P4 */
};
DiaTimingType		kwp2Timing;

//UInt32				_kwp2MinP1				= 0;
//UInt32				_kwp2MaxP1				= 20;
//UInt32				_kwp2MinP2				= 25;
//UInt32				_kwp2MaxP2				= 60;
//UInt32				_kwp2MinP3				= 110;
//UInt32				_kwp2MaxP3				= 5000;
//UInt32				_kwp2MinP4				= 0;
//UInt32				_kwp2MaxP4				= 20;

/*** API implementation ***/

#define MSEC2TICKS(a)			(((a)*_kwp2TicksPerSecond)/1000)
//#define DIA_SLEEP_MSEC(a)		SysTaskDelay(((a)*_kwp2TicksPerSecond)/1000)

/* internal functions decls */
DiaReturnType		_Kwp2TryToSend() KWP2_SECT;
DiaReturnType		_Kwp2TryToRecv() KWP2_SECT;
DiaReturnType		_Kwp2Error(Char *msg, Err err, Boolean fatal) KWP2_SECT; // always return DIA_ERROR
DiaReturnType		_Kwp2OnIdle() KWP2_SECT;
DiaReturnType		_Kwp2ClosePort() KWP2_SECT;
DiaReturnType		_Kwp2OnRecv() KWP2_SECT;
Int16				_Kwp2Compare(const UInt8 *s1, const UInt8 *s2, UInt8 length) KWP2_SECT;
DiaReturnType		_Kwp2SendAsync(UInt8 *req, UInt8 size, UInt8 recv_to) KWP2_SECT;
DiaReturnType		_Kwp2OpenPort(UInt32 baudRate, Boolean enableFlowControl) KWP2_SECT;


inline void	_Kwp2ClearErrors()
{
	_kwp2Error = false;
}

/* start/stop diagnostic session */
DiaReturnType Kwp2Start(UInt32 mask)
{
	if(_kwp2Running) return DIA_OK;

	diaMod->mask = mask;

    _Kwp2ClearErrors();
	_kwp2ErrorCounter = 0;
	_kwp2Running = true;
	_kwp2Connected = false;
	_kwp2Polling = false;
	_kwp2WaitingFor = DIA_WAIT_NONE;
	_kwp2PortOpen = false;
	_kwp2TicksPerSecond = SysTicksPerSecond();
	_kwp2SampleReady = false;

	_kwp2AutoOff = SysSetAutoOffTime(0);

	return DIA_OK;
}

DiaReturnType Kwp2Stop()
{
	if(!_kwp2Running) return DIA_OK;

	if(_kwp2Polling)
		Kwp2StopSensorsPoll();	// stop polling

	// stop current request
	if(_kwp2WaitingFor == DIA_WAIT_NONE || _kwp2WaitingFor == DIA_WAIT_SEND)
		_kwp2WaitingFor = DIA_WAIT_NONE;
	else
		while(1)
			if(Kwp2Run() != DIA_OK) break;
	_kwp2SampleReady = false;

	Kwp2Ctl(KWP2_CTL_DISCONNECT, NULL);	// close connection synchronously

//	if(_kwp2PortOpen)
//		_Kwp2ClosePort();

	if(_kwp2AutoOff) {
		SysSetAutoOffTime(_kwp2AutoOff);
		_kwp2AutoOff = 0;
	}

	_kwp2Running = false;

	return DIA_OK;
}

/* call this function as frequently as possible */
DiaReturnType Kwp2Run()
{
	DiaReturnType return_event = DIA_OK;

	if(!_kwp2Running) return DIA_OK;

	_kwp2SampleReady = false;

	if(_kwp2WaitingFor != DIA_WAIT_NONE)
	{
		if(_kwp2WaitingFor == DIA_WAIT_SEND)
			return_event = _Kwp2TryToSend();
		else
			return_event = _Kwp2TryToRecv();
	}
	else
	// idle, check what we can do
	if(!_kwp2Connected)
		return_event = Kwp2Ctl(KWP2_CTL_CONNECT, NULL);	// try to connect
	else
	if(_kwp2Polling && diaMod->mask != 0)
		return_event = Kwp2Ctl(KWP2_CTL_POLL, NULL);
	else
		return_event = _Kwp2OnIdle(); // send ping to ECU
	return return_event;
}

/* driver states */
Boolean Kwp2IsRunning()
{
	return _kwp2Running;
}

Boolean Kwp2IsConnected()
{
	return _kwp2Connected;
}

Boolean Kwp2IsPolling()
{
	return _kwp2Polling;
}

Char* Kwp2GetLastError() /* return NULL if no errors */
{
	if(!_kwp2Error) return NULL;
	return _kwp2LastError;
}

DiaReturnType Kwp2SetMask(UInt32 mask)
{
	diaMod->mask = mask;
	return DIA_OK;
}

/* polling */
DiaReturnType Kwp2StartSensorsPoll()
{
	if(!_kwp2Running) return DIA_ERROR;
	_kwp2Polling = true;
	_kwp2SampleReady = false;
	return DIA_OK;
}

DiaReturnType Kwp2StopSensorsPoll()
{
	if(!_kwp2Running) return DIA_ERROR;
	_kwp2Polling = false;
	return DIA_OK;
}

DiaSampleType* Kwp2GetSample()	/* return NULL if sample is not ready,
								flush sample */
{
#ifndef EMULATE_ECU_ANSWERS
	if(!_kwp2Polling) return NULL;
	if(!_kwp2SampleReady) return NULL;
	_kwp2SampleReady = false;
#else
	_kwp2SampleP = (DiaSampleType *)&_kwp2Req.sample;
	_kwp2SampleP->time = TimGetSeconds();
	_kwp2SampleP->sensorPresent = 0xFFFFFFFF;
	_kwp2SampleP->data[9]++;
#endif
	return _kwp2SampleP;
}


/* operations */
DiaReturnType Kwp2Ctl(UInt8 ctl, void *param)
{
	DiaCtlType ctlP = NULL;
    DiaReturnType r;

	if(_kwp2WaitingFor != DIA_WAIT_NONE) return _Kwp2Error("Kwp2Ctl(): driver busy", errNone, false);

	_Kwp2ClearErrors();
	_kwp2SampleReady = false;

	ctlP = diaMod->ctl[ctl];
	if(ctlP == NULL)
		return _Kwp2Error("null ctl called", errNone, false);

	_kwp2Req.ctl = ctl;
	_kwp2Req.ctl_param = param;

	r = ctlP(ctl, DIA_SEND, param);
	if(r != DIA_OK) return r;
	r =_Kwp2TryToSend();
	if(r != DIA_OK) return r;
	r = _Kwp2TryToRecv();
	if(r != DIA_OK) return r;

	return DIA_OK;
}

DiaReturnType _Kwp2OnRecv()
{
	DiaCtlType ctlP = NULL;
	
	ctlP = diaMod->ctl[_kwp2Req.ctl];
	return ctlP(_kwp2Req.ctl, DIA_RECV, _kwp2Req.ctl_param);
}

Boolean Kwp2CtlGetResult(UInt8 *ctl, void **param,
						UInt16 *size)	/* return FALSE if result
										is not ready */
{
	return _Kwp2Error("Not implemented", errNone, true);
}


/*** internal functions implementation ***/

DiaReturnType _Kwp2TryToSend()
{
    if(_kwp2WaitingFor != DIA_WAIT_SEND) return DIA_OK;

	if(TimGetTicks() - _kwp2LastReplyTicks >= MSEC2TICKS(diaMod->timing->minP3))
	{
		{
			Err err;

			SrmReceiveFlush(_kwp2PortId, 0);
			SrmSend(_kwp2PortId, _kwp2Req.send, _kwp2Req.send_size, &err);
			if(err != errNone)
				return _Kwp2Error("SrmSend()", err, true);

            _kwp2Req.send_ticks = TimGetTicks();
		}

		_kwp2WaitingFor = DIA_WAIT_ECHO;
	}
	return DIA_OK;
}

Int16 _Kwp2Compare(const UInt8 *s1, const UInt8 *s2, UInt8 length)
{
	UInt8 i;
	Int16 r = 0;
	for(i = 0; i < length; i++)
		if((r = *s1++ - *s2++)) return r;
	return r;
}


DiaReturnType _Kwp2TryToRecv()
{
	Err err;
	UInt32 bytes;
	UInt32 bytes_expected;
	UInt32 timeout;

	switch(_kwp2WaitingFor)
	{
	case DIA_WAIT_ECHO:
		bytes_expected = _kwp2Req.send_size;
		timeout = MSEC2TICKS(diaMod->timing->maxP2);
		break;
	case DIA_WAIT_HDR:
		bytes_expected = 4;
		timeout = MSEC2TICKS(diaMod->timing->maxP2 + 4*diaMod->timing->maxP1);
		break;
	case DIA_WAIT_REPLY:
		bytes_expected = _kwp2Req.recv_size + 1;
		timeout = MSEC2TICKS(diaMod->timing->maxP2 + (bytes_expected + 4)*diaMod->timing->maxP1);
		break;
	default:
		return DIA_OK;
	}

	err = SrmReceiveCheck(_kwp2PortId, &bytes);
	if(err != errNone)
		return _Kwp2Error("SrmReceiveCheck()", err, true);

	if(bytes >= bytes_expected)
	{
		switch(_kwp2WaitingFor)
		{
		case DIA_WAIT_ECHO:
			// recv echo
			SrmReceive(_kwp2PortId, _kwp2Req.recv, bytes_expected, 1, &err);
			if(err != errNone)
				return _Kwp2Error("SrmReceive()", err, true);
			// compare
			if(_Kwp2Compare(_kwp2Req.send, _kwp2Req.recv, _kwp2Req.send_size) != 0)
				return _Kwp2Error("Invalid echo", errNone, true);
			// go to the next step
			_kwp2WaitingFor = DIA_WAIT_HDR;
			return _Kwp2TryToRecv();
		case DIA_WAIT_HDR:
			// get packet length
			SrmReceive(_kwp2PortId, _kwp2Req.recv, 3, 1, &err);
			if(err != errNone)
				return _Kwp2Error("SrmReceive()", err, true);

			_kwp2Req.recv_size = 0x3F & (_kwp2Req.recv_checksum = _kwp2Req.recv[0]);
			_kwp2Req.recv_checksum += _kwp2Req.recv[1];
			_kwp2Req.recv_checksum += _kwp2Req.recv[2];

            if(_kwp2Req.recv_size == 0)
            {
				SrmReceive(_kwp2PortId, _kwp2Req.recv, 1, 1, &err);
				if(err != errNone)
					return _Kwp2Error("SrmReceive()", err, true);
    			_kwp2Req.recv_checksum += _kwp2Req.recv[0];
	            _kwp2Req.recv_size = _kwp2Req.recv[0];
            }
			// go to the next step
			_kwp2WaitingFor = DIA_WAIT_REPLY;
			return _Kwp2TryToRecv();
		case DIA_WAIT_REPLY:
			// read reply
			SrmReceive(_kwp2PortId, _kwp2Req.recv + _kwp2Req.recv_to, _kwp2Req.recv_size+1, 1, &err);
			if(err != errNone)
				return _Kwp2Error("SrmReceive()", err, true);

			// store reply ticks
			_kwp2LastReplyTicks = TimGetTicks();

			// check checksum
			{
				UInt8 i;
				for(i = 0; i < _kwp2Req.recv_size; i++)
					_kwp2Req.recv_checksum += _kwp2Req.recv[_kwp2Req.recv_to + i];
			}
			if(_kwp2Req.recv_checksum != _kwp2Req.recv[_kwp2Req.recv_to + _kwp2Req.recv_size])
				return _Kwp2Error("Bad checksum", errNone, true);

			// check answer
			if(_kwp2Req.recv[_kwp2Req.recv_to + 0] != (_kwp2Req.send[3] | 0x40))
			{

				if(_kwp2Req.recv[_kwp2Req.recv_to + 0] != 0x7F)
					return _Kwp2Error("Unknown answer", errNone, true);
				if(_kwp2Req.recv_size != 3)
					return _Kwp2Error("Bad err answer", errNone, true);
				if(_kwp2Req.recv[_kwp2Req.recv_to + 1] != _kwp2Req.send[3])
					return _Kwp2Error("Bad err answer ID", errNone, true);

				switch(_kwp2Req.recv[_kwp2Req.recv_to + 2])
				{
				case 0x10:
					return _Kwp2Error("ECU: General reject", errNone, false);
				case 0x11:
					return _Kwp2Error("ECU: Service not supported", errNone, false);
				case 0x12:
					return _Kwp2Error("ECU: Subfunction not supported", errNone, false);
				case 0x31:
					return _Kwp2Error("ECU: Request out of range", errNone, false);
				case 0x72:
					return _Kwp2Error("ECU: Transfer aborted", errNone, false);
				case 0x77:
					return _Kwp2Error("ECU: Block checksum error", errNone, false);
				case 0x21: // busy - retry later
					_kwp2Req.retry++;
					_kwp2WaitingFor = DIA_WAIT_SEND;
					return _Kwp2TryToSend();
				}
		   	}

			// clear WaitingFor
			_kwp2WaitingFor = DIA_WAIT_NONE;
			// call OnRecv();
			_kwp2ErrorCounter = 0;
			return _Kwp2OnRecv();
		default:
			return DIA_ERROR;
		}
	}
	else
		if(TimGetTicks() - _kwp2Req.send_ticks > timeout)
		{
			if(_kwp2Req.retry >= 5) 
				return _Kwp2Error("Recv timeout", errNone, false);
			else
			{
				_kwp2Req.retry++;
//				if(bytes > 0)
//					SrmReceiveFlush(_kwp2PortId, 0);
				_kwp2WaitingFor = DIA_WAIT_SEND;
				return _Kwp2TryToSend();
			}
		}

	return DIA_OK;
}

DiaReturnType _Kwp2OnIdle()
{
	if(_kwp2WaitingFor == DIA_WAIT_NONE &&
		(TimGetTicks() - _kwp2LastReplyTicks) > MSEC2TICKS(diaMod->timing->maxP3 >> 1))
		return Kwp2Ctl(KWP2_CTL_PING, NULL);
	return DIA_OK;
}

DiaReturnType _Kwp2Error(Char *msg, Err err, Boolean fatal) // always return true
{
	// check error mode
	if(_kwp2ErrorMode) return DIA_ERROR;

	// enter error mode
	_kwp2ErrorMode = true;

	// reset internal states
	_kwp2WaitingFor = DIA_WAIT_NONE;

//	if(fatal)
//		Kwp2Ctl(DIA_CTL_DISCONNECT, NULL, 0);

	_kwp2ErrorCounter++;
	if(_kwp2ErrorCounter > 6)
	{
		_kwp2ErrorCounter = 0;
		Kwp2Ctl(KWP2_CTL_RECONNECT, NULL);
	}

	// set error
	StrCopy(_kwp2LastError, msg);
	if(err != errNone)
	{
		Char buf[64];

		if(SysErrString(err, buf, 64) != NULL)
		{
			StrCat(_kwp2LastError, ": ");
			StrCat(_kwp2LastError, buf);
		}
	}
	_kwp2Error = true;

	if(_kwp2PortOpen)
	{
		// flush input/output buffer
		SrmReceiveFlush(_kwp2PortId, 0);
		SrmSendFlush(_kwp2PortId);
		// clear Srm errors
		SrmClearErr(_kwp2PortId);
	}

	// leave error mode
	_kwp2ErrorMode = false;

//	Kwp2Ctl(DIA_CTL_ERRORTEST, NULL);

	return DIA_ERROR;//_Kwp2Error("Not implemented", errNone, true);
}

DiaReturnType _Kwp2SendAsync(UInt8 *req, UInt8 size, UInt8 recv_to)
{
	UInt8 checksum = 0;
	UInt8 i;

	if(size > 63)
		return _Kwp2Error("_Kwp2SendAsync(): Can't send long packet", errNone, false);

	checksum += _kwp2Req.send[0] = 0x80 + size;
	checksum += _kwp2Req.send[1] = 0x10;
	checksum += _kwp2Req.send[2] = 0xF1;
	for(i = 0; i < size; i++)
		checksum += _kwp2Req.send[i+3] = req[i];
	_kwp2Req.send[size+3] = checksum;
	_kwp2Req.send_size = size+4;
	_kwp2Req.recv_to = recv_to;

	_kwp2Req.retry = 0;
	_kwp2WaitingFor = DIA_WAIT_SEND;

	return DIA_OK;
}

DiaReturnType _Kwp2OpenPort(UInt32 baudRate, Boolean enableFlowControl)
{
	Err err;

	if(_kwp2PortOpen) return DIA_OK;

	err = SrmOpen(serPortCradleRS232Port, baudRate, &_kwp2PortId);
	if(err != errNone)
		err = SrmOpen(serPortCradlePort, baudRate, &_kwp2PortId);
	if(err != errNone)
		return _Kwp2Error("SrmOpen()", err, true);

	_kwp2PortOpen = true;
		
	if(!enableFlowControl) {
		Int16 paramSize;
		UInt32 flags =	srmSettingsFlagBitsPerChar8 | srmSettingsFlagStopBits1;
		paramSize = sizeof(flags);
		err = SrmControl(_kwp2PortId, srmCtlSetFlags, &flags, &paramSize);
		if(err != errNone)
			return _Kwp2Error("SrmControl()", err, true);
	}

	return DIA_OK;
}

DiaReturnType _Kwp2ClosePort()
{
	Err err;

	if(!_kwp2PortOpen) return DIA_OK;

	_kwp2PortOpen = false;
	err = SrmClose(_kwp2PortId);
	if(err != errNone)
		return _Kwp2Error("SrmClose()", err, false);

	return DIA_OK;
}

/*** ctl handlers ***/

DIA_DEFINE_CTL(Kwp2Connect)
{
	if(op == DIA_SEND)
	{
		DiaReturnType r;
		Err err;
	    UInt8 zero = 0;
//  	  UInt16 pulse_ticks;
		UInt32 rate = 10400;
		UInt16 rate_size = sizeof(rate);
		UInt8 req[] = { 0x81 };

		if(_kwp2Connected) return DIA_RESULT;

		r = _Kwp2OpenPort(360, false);
		if(r != DIA_OK) return r;

	    // do fast init
    	// send 25ms pulse
	    err = SrmSend(_kwp2PortId, &zero, 1, &err);
    	if(err != errNone)
    		SrmClearErr(_kwp2PortId);

		err = SrmReceive(_kwp2PortId, &zero, 1, 10, &err);
    	if(err != errNone)
    		SrmClearErr(_kwp2PortId);

		err = SrmControl(_kwp2PortId, srmCtlSetBaudRate, &rate, &rate_size);
		if(err != errNone)
			return _Kwp2Error("SrmControl()", err, true);

		return _Kwp2SendAsync(req, 1, 0);
	}
	// check result
	if(_kwp2Req.recv_size != 3)
		return _Kwp2Error("Inv. answer length", errNone, true);
	if(_kwp2Req.recv[1] != 0x6B || _kwp2Req.recv[2] != 0x8F)
		return _Kwp2Error("Inv. answer bytes", errNone, true);

	_kwp2Connected = true;

	return DIA_RESULT;
}

DIA_DEFINE_CTL(Kwp2Disconnect)
{
	if(op == DIA_SEND)
	{
		UInt8 req[] = { 0x82 };

		// do disconnect
		if(_kwp2Connected)
		{
			_Kwp2SendAsync(req, 1, 0);

			while(1)
				if(Kwp2Run() != DIA_OK) break;

			_kwp2Connected = false;
		}

		_Kwp2ClosePort();
	}
	return DIA_RESULT;
}

DIA_DEFINE_CTL(Kwp2Reconnect)
{
	if(op == DIA_SEND)
	{
		_kwp2Connected = false;
		_Kwp2ClosePort();
	}
	return DIA_RESULT;
}

DIA_DEFINE_CTL(Kwp2Poll)
{
	UInt8 req1[] = { 0x21, 0x02 };
	UInt8 req2[] = { 0x21, 0x03 };

	if(op == DIA_SEND)
	{
		if((diaMod->mask & 1) != 0)
			return _Kwp2SendAsync(req1, 2, 0);
		else if(diaMod->mask & 2)
			return _Kwp2SendAsync(req2, 2, 28);
		else
			return DIA_OK;
	}
	if(_kwp2Req.recv[1] == 0x02 && (diaMod->mask & 2) != 0)
		return _Kwp2SendAsync(req2, 2, 28);
	_kwp2SampleP = (DiaSampleType *)&_kwp2Req.sample;
	_kwp2SampleP->time = TimGetSeconds();
	_kwp2SampleP->sensorPresent = diaMod->mask;
	_kwp2SampleReady = true;
	return DIA_RESULT;
}

DIA_DEFINE_CTL(Kwp2Ping)
{
	UInt8 req[] = { 0x3e, 0x01 };
	if(op == DIA_SEND)
		return _Kwp2SendAsync(req, 2, 0);
	return DIA_OK;
}

DIA_DEFINE_CTL(Kwp2ReadIdent)
{
	return _Kwp2Error("Not implemented", errNone, true);
}

DIA_DEFINE_CTL(Kwp2ClearErrors)
{
	return _Kwp2Error("Not implemented", errNone, true);
}

DIA_DEFINE_CTL(Kwp2ReadErrors)
{
	return _Kwp2Error("Not implemented", errNone, true);
}

DIA_DEFINE_CTL(Kwp2EcuReset)
{
	return _Kwp2Error("Not implemented", errNone, true);
}
