/* 
 *  $Id: mmcd.c,v 1.23 2003/11/26 15:54:14 kogan Exp $
 *
 *  Copyright (c) 2003, Dmitry Yurtaev <dm1try@umail.ru>
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#include <PalmOS.h>

#ifdef HANDSPRING_SUPPORT
#include <HsExt.h>
#endif

#include "config.h"
#include "dia.h"
#include "mmcd.h"
#include "graph.h"
#include "panel.h"
#include "selector.h"
#include "dbg.h"

//
UInt32 screenWidth, screenHeight, screenDepth;
Int16 screenDensity;
MemHandle bigFontH;
FontPtr bigFontP;

// global state vars
Boolean colorMode = false;
UInt32 ticksPerSecond;

Boolean talkingECU = false;
UInt8 currentSensor = 0;
//UInt8 currentBank = 0;
UInt8 currentMode = 0;

#define MODE_MASK			0x0F
#define REVIEW_MODE			0x00
#define MONITOR_MODE		0x01

#define SUBMODE_MASK		0xF0
#define BANK_SETUP_SUBMODE	0x80



UInt8	sensorIndex[32] = {
	 1,  2,  3,  4,  5,  6,  7,	 8,
     9, 10, 11, 12, 13, 14, 15,	16,
    17, 18, 19, 20, 21, 22, 23,	24,
    25, 26, 27, 28, 29, 30, 31, 32
};
UInt8	sensorColor[32] = {
	0x71, 0x6e, 0x05, 0x38, 0xca, 0x3b, 0xa4, 0x39,
	0x6f, 0x20, 0x5f, 0x94, 0x03, 0x68, 0xa5, 0xad,
	0x82, 0x98, 0xc3, 0x27, 0x47, 0xc9, 0x1b, 0x44,
	0x29, 0x32, 0x35, 0x57, 0x93, 0x9b, 0xdb, 0xce,
};
UInt32	sensorCaptureMask = 0;
UInt32	sensorGraphMask = 0;
UInt8	sensorSelection = 0;



//UInt32 graphSensors = 0;
//UInt32 captureSensors = 0;
DiaSampleType currentSample;
FileHand stream;
Char streamName[32];
UInt16 streamAttr;

Char editFormTitle[32];

Int32 alarmFreq[5] = { 0, 4000, 1000, 300, 50 };

// custom widgets
GraphType graph;
PanelType panel;
SelectorType selector;
Coord graphX, graphY, graphW, graphH;

#define MIN(a,b) ((a)<(b)?(a):(b))
#define MAX(a,b) ((a)>(b)?(a):(b))

//#define ECU_DEBUG
#ifndef ECU_DEBUG
#define ECU_BAUDRATE 1920
#else
#define ECU_BAUDRATE 9600
#endif

#define offsetof(s,m)   ((UInt16)&(((s *)0)->m))

/*
 *
 */

typedef struct _ModPrefs {
	DiaPrefsType	custom;		/* custom module prefs */

	UInt32			sensorCaptureMask;
	UInt32			sensorGraphMask;
	UInt32			capture;
	UInt32			graph;
	UInt8			currentBank;
	UInt8			currentMode;
	UInt8			sensorSelection;
	UInt32			polarity;
	UInt8			threshold[SENSOR_COUNT];
	UInt8			sound[SENSOR_COUNT];
	UInt8			sensorIndex[SENSOR_COUNT];
	Boolean			hideUnused;
	GraphMode		graphMode;
	Boolean			audibleAlarms;
	UInt16			topVisible;
	DiaTimingType	timing;
} ModPrefs;

typedef struct Prefs {
	ModPrefs		mod[DIA_NMODS];

	UInt8			metricUnit;
	UInt8			currentMod;
} Prefs;

Prefs prefs;
ModPrefs *modPrefs;
UInt16 topVisible;
UInt8 metricUnit;
UInt8 audibleAlarms;
GraphMode graphMode;	// show sensor numeric value in Monitor mode
						// instead of graph

/*
 *  return pointer to object in form by its identifier, if found
 *  NULL otherwise
 */
void *FrmGetObjectPtrById(FormType *form, UInt16 id)
{
	UInt16 index = FrmGetObjectIndex(form, id);
	return index == frmInvalidObjectId ? NULL : FrmGetObjectPtr(form, index);
}

/*
 *  graph-callback stream reader
 */

Int32 streamReader(Int32 streamOffset, DiaSampleType *pBuffer, Int32 sampleCount, UInt16 sampleSize)
{
	Int32 samplesRead = 0;
	Err err;

	// stream length request
	if(pBuffer == NULL) {
		FileTell(stream, &samplesRead, &err);
		return samplesRead / sampleSize;
	}

	if(sampleCount == 0) return 0;

	FileSeek(stream, streamOffset * sampleSize, fileOriginBeginning);

	samplesRead = FileRead(stream, pBuffer, sampleSize, sampleCount, &err);
	if(samplesRead < sampleCount)
		MemSet(&pBuffer[samplesRead], (sampleCount - samplesRead) * sampleSize, 0);
	return samplesRead;
}

Int32 streamWriter(DiaSampleType *sample, UInt16 sampleSize)
{
	Int32 samplesWritten = 0;
	Err err;

	FileSeek(stream, 0, fileOriginEnd);
	samplesWritten = FileWrite(stream, sample,
		sampleSize, 1, &err);
	return samplesWritten;
}

/*
 *  review mode-specific event handler
 */

Int32 scrollerScale = 1;
void _setScroller(Int32 value) {
	ScrollBarType *bar = FrmGetObjectPtrById(FrmGetActiveForm(), scrollBarId);
	scrollerScale = graph.length / 32000 + 1;
	SclSetScrollBar(bar, value / scrollerScale, 0, 
		graph.length > graph.width
		? (graph.length - graph.width) / scrollerScale
		: 0,
		graph.width / scrollerScale
	);
}

void updateSampleTime()
{
	Char s[32];
	DateTimeType date;

	TimSecondsToDateTime(currentSample.time, &date);
	StrPrintF(s, "%02d:%02d:%02d", date.hour, date.minute, date.second);
	FrmCopyLabel(FrmGetActiveForm(), timeLabelId, s);
}

void _onSelectorSelect()
{
	FormType *form = FrmGetActiveForm();
	UInt8 sensor = selector.selection;
	const Char *label = diaMod->sensor[sensor].exists ?
		diaMod->sensor[sensor].desc :
		"";
	UInt16 scrollmax = diaMod->sensorCount/4 + (diaMod->sensorCount%4 ? 1 : 0);
	FrmCopyLabel(form, sensorLabelId, label);

	SclSetScrollBar(FrmGetObjectPtrById(form, bankSetupScrollId),
		selector.pos, 0,
		scrollmax < 6 ? 0 : scrollmax-6, 6);
}

void _setSensorGraphMask(UInt32 mask)
{
//	dbgInfo("aaa");
	if(sensorGraphMask == mask) return;
//	dbgInfo("bbb");
	sensorGraphMask = mask;
	PnlRefresh(&panel);
	GrfOnSensorGraphMaskChange(&graph);
}

void _setSensorCaptureMask(UInt32 mask)
{
	if(sensorCaptureMask == mask) return;
	sensorCaptureMask = mask;
	diaMod->DiaSetMask(mask);
	PnlRefresh(&panel);
	GrfOnSensorCaptureMaskChange(&graph);
}

void _setSensorSelection(UInt8 selection)
{
	if(sensorSelection == selection) return;
	sensorSelection = selection;
	PnlRefresh(&panel);
	GrfOnSensorSelectionChange(&graph);
	if(currentMode & BANK_SETUP_SUBMODE)
	{
       	SlSetSelection(&selector, sensorIndex[sensorSelection]);
		SlEnsureVisible(&selector);
		_onSelectorSelect();
	}
}

void _setSensorIndex(UInt8 index, UInt8 sensor)
{
	UInt8 i;
	Boolean dup = false;
	UInt32 mask = 0;

	for(i = 0; i < 32; i++)
		if(i != index && sensorIndex[i] == sensor)
		{
			dup = true;
			mask |= 1L << i;
		}

	sensorIndex[index] = sensor;

	if(dup)
	{
		if((sensorGraphMask & mask) != 0)
			_setSensorGraphMask(sensorGraphMask | mask | (1L << index));
		else
			_setSensorGraphMask(sensorGraphMask & ~(mask | (1L << index)));
	}
	PnlRefresh(&panel);
	if((currentMode & BANK_SETUP_SUBMODE) != 0)
		SlRedraw(&selector);
}

Boolean reviewHandleEvent(EventType* e)
{
	if(GrfHandleEvent(&graph, e)) return true;

	switch(e->eType) {

	case sclRepeatEvent:
		// user moved scroller - update graph
		if(e->data.sclRepeat.newValue != e->data.sclRepeat.value)
			GrfSetGraph(&graph, graph.length, e->data.sclRepeat.newValue * scrollerScale, graph.cursor);
		break;

	case grfChangeEvent:
		// graph has been changed by user interaction
		{
			GraphEventType *event = (GraphEventType *)e;

			// position changed? update scroller, then
			if(event->data.grfChange.positionChanged || event->data.grfChange.lengthChanged) {
				_setScroller(event->data.grfChange.position);
			}

			// cursor moved? update values and currectSample
			if(event->data.grfChange.cursorChanged) {
				currentSample = (*((DiaSampleType*)&graph._buffer[(graph.cursor - graph.position)*graph._sampleSize]));
				/* Insert SP 06/01/03 - update hidden sensors */
				PnlRefresh(&panel);
				/* End Insert */
				PnlUpdate(&panel, &currentSample);
				updateSampleTime();
			}
		}
		return true;

	/*
	 * removed, pageUp and pageDown now scroll thru banks
	 */
	
	case keyDownEvent:

		// move cursor 1 sample left if not at the beginning of graph
		if(e->data.keyDown.chr == hard1Chr && graph.cursor > 0) {
			Int32 newPosition = graph.position;
			Int32 newCursor = graph.cursor - 1;

			SndPlaySystemSound(sndClick);

			// update graph and scroller positions
			// if moved out of visible area
			if(newCursor < newPosition) {
				_setScroller(newPosition);
				newPosition = newCursor;
			}

			// update the graph
			GrfSetGraph(&graph, graph.length, newPosition, newCursor);

			// update values
			currentSample = (*((DiaSampleType*)&graph._buffer[(graph.cursor - graph.position)*graph._sampleSize]));
			// Insert SP 06/01/03 - update hidden sensors
			PnlRefresh(&panel);
			// End Insert
			PnlUpdate(&panel, &currentSample);
			updateSampleTime();
			return true;
		}

		// move cursor 1 sample right if not at the end of graph
		if(e->data.keyDown.chr == hard2Chr && graph.cursor < graph.length - 1) {
			Int32 newPosition = graph.position;
			Int32 newCursor = graph.cursor + 1;

			SndPlaySystemSound(sndClick);

			// update graph and scroller positions
			// if moved out of visible area
			if(newCursor >= newPosition + graph.width) {
				newPosition = newCursor - graph.width + 1;
				_setScroller(newPosition);
			}

			// update the graph
			GrfSetGraph(&graph, graph.length, newPosition, newCursor);

			// update values
			currentSample = (*((DiaSampleType*)&graph._buffer[(graph.cursor - graph.position)*graph._sampleSize]));
			// Insert SP 06/01/03 - update hidden sensors
			PnlRefresh(&panel);
			// End Insert
			PnlUpdate(&panel, &currentSample);
			updateSampleTime();
			return true;
		}
		break;

	default:
		break;
	}
	return false;
}

/*
 *  monitor mode-specific event handler
 */

 
Boolean monitorHandleEvent(EventType* e)
{
	FormType *form = FrmGetActiveForm();
	static UInt32 startTicks;
	static UInt32 cycles;
	static UInt32 lastUpdateTime;
	static UInt32 lastPanelUpdateTime;
	static Boolean needsUpdate;
	DiaSampleType *sampleP = NULL;
	Char *lastErrorP = NULL;
	static Boolean logging;

	GrfHandleEvent(&graph, e);

	if(e->eType == keyDownEvent && !(e->data.keyDown.modifiers & autoRepeatKeyMask)) {
		switch(e->data.keyDown.chr) {
		case hard1Chr:
			// change sensor for numeric display
			SndPlaySystemSound(sndClick);
			if(currentMode & BANK_SETUP_SUBMODE) return true;

			// next available
	        if(graph._mode == GRAPH_NUMERIC && sensorCaptureMask)
    	    {
				UInt8 index = sensorSelection;
				do {
					index = (index + 1) % 32;
				} while((diaMod->sensor[sensorIndex[index]].mask & sensorCaptureMask) == 0);
				_setSensorSelection(index);
			}
			return true;

		case hard2Chr:
			// toggle Graph checkbox
			SndPlaySystemSound(sndClick);
			if(currentMode & BANK_SETUP_SUBMODE) return true;
			{
				UInt16 index = FrmGetObjectIndex(form, graphCheckId);
				Int16 newValue = FrmGetControlValue(form, index) ? 0 : 1;
				FrmSetControlValue(form, index, newValue);
				GrfSetMode(&graph, graphMode = newValue ? GRAPH_NORMAL : GRAPH_NUMERIC );
				GrfAppendSample(&graph, NULL);

				PnlSetMode(&panel, graphMode != GRAPH_NORMAL ? PANEL_SINGLESELECT_VAL : PANEL_MULTISELECT);
			}
			return true;

		case hard3Chr:
			// toggle Log checkbox
			SndPlaySystemSound(sndClick);
			if(currentMode & BANK_SETUP_SUBMODE) return true;
			{
				UInt16 index = FrmGetObjectIndex(form, logCheckId);
				Int16 newValue = FrmGetControlValue(form, index) ? 0 : 1;
				FrmSetControlValue(form, index, newValue);

				logging = newValue;
			}
			return true;

		case hard4Chr:
			// toggle Pause checkbox
			SndPlaySystemSound(sndClick);
			if(currentMode & BANK_SETUP_SUBMODE) return true;
			{
				UInt16 index = FrmGetObjectIndex(form, pauseCheckId);
				Int16 newValue = FrmGetControlValue(form, index) ? 0 : 1;
				FrmSetControlValue(form, index, newValue);
				
				if(newValue) {
					RectangleType r = { { 1, 147 }, { 116, 13 } };

					WinEraseRectangle(&r, 0);
					FrmShowObject(form, FrmGetObjectIndex(form, reviewButtonId));
					FrmShowObject(form, FrmGetObjectIndex(form, doneButtonId));
					diaMod->DiaStopSensorsPoll();
					diaMod->DiaStop();
					talkingECU = false;
					GrfAppendSample(&graph, NULL);
				} else {
					FrmHideObject(form, FrmGetObjectIndex(form, reviewButtonId));
					FrmHideObject(form, FrmGetObjectIndex(form, doneButtonId));
					// update current mode pushbutton as its frame
					// gets erased by neighbour hiding buttons
					FrmShowObject(form, FrmGetObjectIndex(form, monitorButtonId));
					diaMod->DiaStart(sensorCaptureMask);
					diaMod->DiaStartSensorsPoll();
//					openPort(serialSpeed, false);
					talkingECU = true;
					startTicks = TimGetTicks();
					cycles = 0;
				}
			}
			return true;
		}
	}
	
	/* Modified SP 06/01/03 - change conditional structure */
	if(e->eType == ctlSelectEvent) {
		switch (e->data.ctlSelect.controlID) {
		case pauseCheckId:
			// disable mode-switching when serial communication
			// is in progress
			if(e->data.ctlSelect.on) {
				RectangleType r = { { 1, 147 }, { 116, 13 } };

				WinEraseRectangle(&r, 0);
				FrmShowObject(form, FrmGetObjectIndex(form, reviewButtonId));
				FrmShowObject(form, FrmGetObjectIndex(form, doneButtonId));
				diaMod->DiaStopSensorsPoll();
				diaMod->DiaStop();
				talkingECU = false;
				GrfAppendSample(&graph, NULL);
			} else {
				FrmHideObject(form, FrmGetObjectIndex(form, reviewButtonId));
				FrmHideObject(form, FrmGetObjectIndex(form, doneButtonId));
				// update current mode pushbutton as its frame
				// gets erased by neighbour hiding buttons
				FrmShowObject(form, FrmGetObjectIndex(form, monitorButtonId));
				diaMod->DiaStart(sensorCaptureMask);
				diaMod->DiaStartSensorsPoll();
				talkingECU = true;
				startTicks = TimGetTicks();
				cycles = 0;
			}
			break;

		case logCheckId:
			logging = e->data.ctlSelect.on;
			break;
              
		case graphCheckId:
			GrfSetMode(&graph, graphMode = e->data.ctlSelect.on ? GRAPH_NORMAL : GRAPH_NUMERIC );
			GrfAppendSample(&graph, NULL);
			PnlSetMode(&panel, graphMode != GRAPH_NORMAL ? PANEL_SINGLESELECT_VAL : PANEL_MULTISELECT);
			break;
		}
	}
        /* End Modified - change conditional structure */

#ifdef EMULATE_ECU_ANSWERS
	if(diaMod->DiaIsRunning())
	{
	   	sampleP = diaMod->DiaGetSample();
   		if(sampleP != NULL)
	   	{
			// show it
			GrfAppendSample(&graph, sampleP);
			needsUpdate = true;
			cycles++;

			// if log checkbox is checked - write the sample to log file
			if(logging) streamWriter(sampleP, offsetof(DiaSampleType,data) + diaMod->sampleDataSize);

			if(TimGetTicks() > lastPanelUpdateTime + ticksPerSecond / 6) {
				lastPanelUpdateTime = TimGetTicks();
				PnlUpdate(&panel, sampleP);
			}
   		}
   	}
#endif

    switch(diaMod->DiaRun())
    {
    case DIA_RESULT:		// maybe got sample
    	sampleP = diaMod->DiaGetSample();
    	if(sampleP != NULL)
    	{
			// show it
			GrfAppendSample(&graph, sampleP);
			needsUpdate = true;
			cycles++;

			// if log checkbox is checked - write the sample to log file
			if(logging) streamWriter(sampleP, offsetof(DiaSampleType,data) + diaMod->sampleDataSize);

			if(TimGetTicks() > lastPanelUpdateTime + ticksPerSecond / 6) {
				lastPanelUpdateTime = TimGetTicks();
				PnlUpdate(&panel, sampleP);
			}

/*			// check alarm threshold
			if(audibleAlarms)
			{
				UInt8 sensor;
				for(sensor = 0; sensor < 32; sensor++)
				{
					register DiaSensorType *sensorP = &diaMod->sensor[sensor];

					if(sensorP->exists &&
						sensorP->sound &&
						((sampleP->sensorPresent & diaMod->sensor[sensor].mask) != 0))
					{
						Int8 c = sensorP->compare(sensorP, &sampleP->data[sensorP->addr], (UInt8*)&sensorP->threshold);

						if((sensorP->polarity == 0 && c >= 0) ||
							(sensorP->polarity == 1 && c <= 0))
						{
							SndCommandType sndCmd = { sndCmdFrqOn, 0, alarmFreq[sensorP->sound], 100, sndMaxAmp };
							SndDoCmd(NULL, &sndCmd, true);
						}
					}
				}
			}*/
    	}
    	break;
    case DIA_ERROR:
   		lastErrorP = diaMod->DiaGetLastError();
    	break;
    default:
    }


	if(TimGetTicks() > lastUpdateTime + ticksPerSecond / 6 || lastErrorP != NULL) {
		lastUpdateTime = TimGetTicks();

		// show sample rate and current sensor
		if(diaMod->DiaIsRunning()) {
			static Char buf[32];
			Int16 strLen, labelWidth;
			RectangleType r;
			Char *str;

			if(lastErrorP != NULL)
			{
				strLen = StrLen(lastErrorP);
				str = lastErrorP;
			}
			else
			{
				strLen = StrPrintF(buf, "%ldHz", cycles * ticksPerSecond / (lastUpdateTime - startTicks + 1));
				str = buf;
			}	
			labelWidth = FntCharsWidth(str, strLen);

			WinDrawTruncChars(str, strLen, 1, 147, 117);
			if(labelWidth < 116)
			{
				r.topLeft.x = 1 + labelWidth;
				r.topLeft.y = 147;
				r.extent.x = 116 - labelWidth;
				r.extent.y = 13;
				WinEraseRectangle(&r, 0);
			}
		}
	}

	return false;
}

void doAboutDialog()
{
	FormType *form = FrmInitForm(aboutDialogId);
	FrmDoDialog(form);
	FrmDeleteForm(form);
}

/*
 *  p r e f s
 */

void FldInsertText(FieldType *field, const Char *source)
{
	MemHandle textHandle;
	Char *text;

	textHandle = FldGetTextHandle(field);
	if(textHandle != NULL) {
		FldSetTextHandle(field, NULL);
		MemHandleFree(textHandle);
	}

	textHandle = MemHandleNew(5);
	text = MemHandleLock(textHandle);
	StrNCopy(text, source, 5);
	MemHandleUnlock(textHandle);

	FldSetTextHandle (field, textHandle);
}

void doPrefsDialog()
{
/*
	FormType *form = FrmInitForm(prefsDialogId);
	UInt16 i, index;
	FieldType *field;
	Char *s, str[16];
	ControlType* ctl;
	UInt32 serialSpeed = 0;

	for(i = 0; i < 8; i++) {
//		DiaSensorType *sensor = &diaMod->sensor[sensorPosition[currentBank * 8 + i]];
		DiaSensorType *sensor = &diaMod->sensor[sensorPosition[3 * 8 + i]];

		index = FrmGetObjectIndex(form, customSensorId+i*3);	// checkbox
		FrmSetControlValue(form, index, sensor->exists);

		index = FrmGetObjectIndex(form, customSensorId+i*3+1);	// addr
		field = FrmGetObjectPtr(form, index);
		StrIToH(str, sensor->addr);
		FldInsertText(field, str + 6);

		index = FrmGetObjectIndex(form, customSensorId+i*3+2);	// slug
		field = FrmGetObjectPtr(form, index);
		s = sensor->slug;
		FldInsertText(field, s);
	}

	ctl = FrmGetObjectPtrById(form, speedTriggerId); // serial speed
	StrPrintF(str, "%ld bps", serialSpeed);
	CtlSetLabel(ctl, str);

	ctl = FrmGetObjectPtrById(form, unitTriggerId); // unit
	switch(metricUnit) {
	case PREFS_UNIT_ENGLISH: CtlSetLabel(ctl, "English"); break;
	case PREFS_UNIT_NUMERIC: CtlSetLabel(ctl, "Numeric"); break;
	default:		 CtlSetLabel(ctl, "Metric"); break;
	}

	index = FrmGetObjectIndex(form, alarmsCheckId);
	FrmSetControlValue(form, index, audibleAlarms);

	i = FrmDoDialog(form);
	if(i == defaultButtonId) {
//		MemMove(diaSensor, diaMod->sensorDefault, sizeof(diaSensor));

		captureSensors = 0;
		graphSensors = 0;
		metricUnit = PREFS_UNIT_METRIC;
		serialSpeed = ECU_BAUDRATE;
		
		for(i = 0; i < SENSOR_COUNT; i++) {
			if(diaMod->sensor[i].capture) captureSensors |= (1L << i);
			if(diaMod->sensor[i].graph) graphSensors |= (1L << i);
		}
	} else if(i == okButtonId) {
		for(i = 0; i < 8; i++) {
			DiaSensorType *sensor = &diaMod->sensor[sensorPosition[3 * 8 + i]];

			index = FrmGetObjectIndex(form, customSensorId+i*3);	// checkbox
			sensor->exists = FrmGetControlValue(form, index) ? 1 : 0;

			index = FrmGetObjectIndex(form, customSensorId+i*3+1);	// addr
			field = FrmGetObjectPtr(form, index);
			s = FldGetTextPtr(field);
			while(s && *s) {
				index <<= 4;
				if(*s >= '0' && *s <= '9')
					index += *s - '0';
				else if(*s >= 'a' && *s <= 'f')
					index += *s - 'a' + 10;
				else if(*s >= 'A' && *s <= 'F')
					index += *s - 'A' + 10;
				s++;
			}
			sensor->addr = index;

			index = FrmGetObjectIndex(form, customSensorId+i*3+2);	// slug
			field = FrmGetObjectPtr(form, index);
			s = FldGetTextPtr(field);
			StrNCopy(sensor->slug, s ? s : "", 5);
		}
		
		ctl = FrmGetObjectPtrById(form, speedTriggerId); // serial speed
		serialSpeed = StrAToI(CtlGetLabel(ctl));

		ctl = FrmGetObjectPtrById(form, unitTriggerId); // serial speed
		s = (Char*)CtlGetLabel(ctl);
		if(0 == StrCompare(s, "English")) {
			metricUnit = PREFS_UNIT_ENGLISH;
		} else if(0 == StrCompare(s, "Numeric")) {
			metricUnit = PREFS_UNIT_NUMERIC;
		} else {
			metricUnit = PREFS_UNIT_METRIC;
		}

		index = FrmGetObjectIndex(form, alarmsCheckId);
		audibleAlarms = FrmGetControlValue(form, index) ? true : false;
	}

	FrmDeleteForm (form);
	*/
}

/*
 *  a l a r m s
 */

static Char* alarmListSlugs[32];
//static Char alarmListStrings[32*6];
static Char alarmListSensor[32];

void updateAlarmThreshold(FormType* form, UInt16 si)
{
	static Char str[16] = "\0";
//	StrPrintF(str, "%u", diaMod->sensor[si].threshold);
	diaMod->sensor[si].format(&diaMod->sensor[si], (UInt8*)&diaMod->sensor[si].threshold, str);
	FrmCopyLabel(form, threshFieldId, str);
}

void updateAlarmForm(FormType* form, UInt16 si)
{
	UInt16 index;
	ControlType* control;
	UInt16 v[4] = { 0, 
		255, 1, diaMod->sensor[si].threshold };

	index = FrmGetObjectIndex(form, alarmTriggerId);
	control = FrmGetObjectPtr(form, index);
	CtlSetLabel(control, diaMod->sensor[si].slug);

	FrmSetControlGroupSelection(form, soundGroupId, sound0ButtonId + diaMod->sensor[si].sound);
	index = FrmGetObjectIndex(form, threshSliderId);
	control = FrmGetObjectPtr(form, index);
	CtlSetSliderValues(control, &v[0], &v[1], &v[2], &v[3]);

	index = FrmGetObjectIndex(form, threshSignButtonId);
	control = FrmGetObjectPtr(form, index);
	CtlSetLabel(control, diaMod->sensor[si].polarity ? "<" : ">");

	updateAlarmThreshold(form, si);
}

void updateAlarmList(FormType* form)
{
	Int16 i, last = 0;
	ListType* list;
/*
	for(i = 0; i < SENSOR_COUNT; i++) {
		if(diaMod->sensor[sensorPosition[i]].exists) {
			alarmListSensor[last] = sensorPosition[i];
			StrNCopy(alarmListSlugs[last], diaMod->sensor[sensorPosition[i]].slug, 5);
			if(diaMod->sensor[sensorPosition[i]].sound)
				StrNCat(alarmListSlugs[last], "#", 6);
			last++;
		}
	}
*/
	for(i = 0; i < SENSOR_COUNT; i++) {
		if(diaMod->sensor[i].exists) {
			alarmListSensor[last] = i;
			StrNCopy(alarmListSlugs[last], diaMod->sensor[i].slug, 5);
			if(diaMod->sensor[i].sound)
				StrNCat(alarmListSlugs[last], "#", 6);
			last++;
		}
	}

	list = FrmGetObjectPtr(form, FrmGetObjectIndex(form, alarmListId));
	LstSetListChoices(list, alarmListSlugs, last);
}

Boolean alarmsFormHandleEvent(EventType *e)
{
	FormType *form = FrmGetActiveForm();
	SndCommandType sndCmd = { sndCmdFrqOn, 0, 1000, 200, sndMaxAmp };
	static int si;
//	Int32 i;
	Err err;

	if(SysHandleEvent(e) || MenuHandleEvent(NULL, e, &err)) return true;
	
	switch(e->eType) {

	case frmOpenEvent:

//		for(i = 0; i < SENSOR_COUNT; i++)
//			alarmListSlugs[i] = alarmListStrings + i * 6;

//		si = selectedSensor & 0x1f;
//		for(i = 0; i < SENSOR_COUNT; i++) {
//			if(diaMod->sensor[si].exists)
//				break;
//			si = (si + 1) % SENSOR_COUNT;
//		}

//		FrmDrawForm(form);
//		updateAlarmList(form);
//		updateAlarmForm(form, si);

		return true;

	case popSelectEvent:
//		si = alarmListSensor[e->data.popSelect.selection];
//		updateAlarmForm(form, si);
		return true;


	case frmCloseEvent:
		// if user switcher application when this form is open
		// post frmCloseEvent to caller form as it will get it
		// a chance to free resources
		FrmReturnToForm(0);
		return true;

	case ctlRepeatEvent:
		switch(e->data.ctlRepeat.controlID) {
		case threshSliderId:
			diaMod->sensor[si].threshold = e->data.ctlRepeat.value;
			updateAlarmThreshold(form, si);
			return false;
		}
		break;

	case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID) {
		case okButtonId:
			FrmReturnToForm(0);
			return true;

		case sound0ButtonId:
		case sound1ButtonId:
		case sound2ButtonId:
		case sound3ButtonId:
		case sound4ButtonId:
			diaMod->sensor[si].sound = e->data.ctlSelect.controlID - sound0ButtonId;
			updateAlarmList(form);
			if((sndCmd.param1 = alarmFreq[diaMod->sensor[si].sound]))
				SndDoCmd(NULL, &sndCmd, true);
			return true;

		case threshSignButtonId:
			{
				UInt16 index = FrmGetObjectIndex(form, threshSignButtonId);
				ControlType* control = FrmGetObjectPtr(form, index);
				const Char *label = CtlGetLabel(control);
				diaMod->sensor[si].polarity = *label == '<' ? 0 : 1;
				CtlSetLabel(control, *label == '<' ? ">" : "<");
			}
			return true;
		}
		return false;

	default:
		break;
	}

	return false;
}

/*
 *  e d i t
 */

/*
 *  update form according to current mode
 */

Boolean switchMode(UInt8 newMode)
{
	FormType *form = FrmGetActiveForm();
	
	UInt16 pauseCheckIndex = FrmGetObjectIndex(form, pauseCheckId);
	UInt16 logCheckIndex = FrmGetObjectIndex(form, logCheckId);
	UInt16 graphCheckIndex = FrmGetObjectIndex(form, graphCheckId);
	UInt16 timeLabelIndex = FrmGetObjectIndex(form, timeLabelId);
	UInt16 scrollBarIndex = FrmGetObjectIndex(form, scrollBarId);

	Boolean bankSetup = newMode & BANK_SETUP_SUBMODE;
	Boolean review = (newMode & MODE_MASK) == REVIEW_MODE;
	Boolean monitor = (newMode & MODE_MASK) == MONITOR_MODE;

	UInt8 oldMode = currentMode;

	if(newMode == currentMode) return false;

	if((newMode & MODE_MASK) != (oldMode & MODE_MASK))
		diaMod->DiaStop();


	currentMode = newMode;

	{	// erase graph
		RectangleType r = { { 0, graphY - 14}, { 160, 1 } };
		WinEraseRectangle(&r, 0);
		r.topLeft.y = graphY - 12;
		r.extent.y = 11;
		WinEraseRectangle(&r, 0);
	}

	// turn off all controls

	{
		// turn off bank setup controls
//		Int16 i;
//		for(i = 0; i < 9; i++)
//			FrmHideObject(form, FrmGetObjectIndex(form, page1ButtonId+i));
		FrmHideObject(form, FrmGetObjectIndex(form, bankSetupScrollId));
		FrmHideObject(form, FrmGetObjectIndex(form, sensorLabelId));
		SlHideSelector(&selector);
	}

	PnlRefresh(&panel);
	PnlUpdate(&panel, &currentSample);

	PnlSetMode(&panel, bankSetup ? PANEL_SINGLESELECT_SLUG :
		(review ? PANEL_MULTISELECT :
			(graphMode != GRAPH_NORMAL ? PANEL_SINGLESELECT_VAL : PANEL_MULTISELECT)));
	PnlShowPanel(&panel);

	if(!monitor || bankSetup)
	{
		// hide Graph, Log and Pause checkboxes
		FrmHideObject(form, pauseCheckIndex);
		FrmHideObject(form, logCheckIndex);
		FrmHideObject(form, graphCheckIndex);
	}

	if(!review || bankSetup)
	{
		FrmHideObject(form, timeLabelIndex);
		FrmHideObject(form, scrollBarIndex);
	}


	if(monitor && !bankSetup)
	{
		FrmSetControlValue(form, pauseCheckIndex, !diaMod->DiaIsRunning());
		FrmSetControlValue(form, graphCheckIndex, graphMode == GRAPH_NORMAL);
		FrmShowObject(form, pauseCheckIndex);
		FrmShowObject(form, graphCheckIndex);
		FrmShowObject(form, logCheckIndex);

		GrfShowGraph(&graph);
		GrfSetMode(&graph,
			FrmGetControlValue(form, graphCheckIndex) ? GRAPH_NORMAL : GRAPH_NUMERIC);
		if((newMode & MODE_MASK) != (oldMode & MODE_MASK))
			GrfEraseGraph(&graph);
		GrfShowCursor(&graph, 0);
		GrfAppendSample(&graph, 0);
	}

	if(review && !bankSetup)
	{
		FrmShowObject(form, timeLabelIndex);

		// show graph
		GrfShowGraph(&graph);
		GrfSetMode(&graph, GRAPH_NORMAL);
		GrfShowCursor(&graph, 1);
		GrfUpdateGraph(&graph);

		// show graph scroller
		_setScroller(graph.position);
		FrmShowObject(form, scrollBarIndex);
	}

	/* highlight "S" button */
	FrmSetControlValue(form,
		FrmGetObjectIndex(form, bankSetupButtonId), bankSetup ? 1 : 0);

	if(bankSetup)
	{
//		Int16 i;
		// hide graph
		GrfHideGraph(&graph);
//		GrfSetMode(&graph, GRAPH_OFF);
//		GrfEraseGraph(&graph);
//		GrfShowCursor(&graph, 0);
		WinEraseRectangle(&graph.bounds, 0);
//		GrfUpdateGraph(&graph);
//		GrfAppendSample(&graph, NULL);

		// turn on bank setup controls
//		for(i = 0; i < diaMod->sensorCount/28 + (diaMod->sensorCount%28 ? 1 : 0); i++)
		FrmShowObject(form, FrmGetObjectIndex(form, bankSetupScrollId));


//		for(i = 0; i < 16; i++)
//			FrmShowObject(form, FrmGetObjectIndex(form, page1ButtonId+i));
		FrmShowObject(form, FrmGetObjectIndex(form, sensorLabelId));
		SlShowSelector(&selector);
//		_onSelectorSelect();

		SlSetSelection(&selector, sensorIndex[sensorSelection]);
   		SlEnsureVisible(&selector);
		_onSelectorSelect();

	}

	return true;
}

Boolean editFormHandleEvent(EventType *e)
{
	Err err;
	FormType *form = FrmGetActiveForm();

	if(MenuHandleEvent(NULL, e, &err)) return true;

	// give a chance to mode-specific handlers first
	if((currentMode & MODE_MASK) == REVIEW_MODE && reviewHandleEvent(e)) return true;
	if((currentMode & MODE_MASK) == MONITOR_MODE && monitorHandleEvent(e)) return true;
	// if serial comm in progress - block all system events
	// otherwise just block hardware buttons 1..4
	if(!talkingECU && !(
		e->eType == keyDownEvent && 
			(
			e->data.keyDown.chr == hard1Chr ||
			e->data.keyDown.chr == hard2Chr ||
			e->data.keyDown.chr == hard3Chr ||
			e->data.keyDown.chr == hard4Chr
			)
		) && (SysHandleEvent(e) || MenuHandleEvent(NULL, e, &err))) return true;

	// should be the last as it interfere with MenuHandleEvent
	if(PnlHandleEvent(&panel, e)) return true;
	if(SlHandleEvent(&selector, e)) return true;

	switch(e->eType) {

	case frmOpenEvent:
		{

			FrmSetTitle(form, editFormTitle);
			FrmDrawForm(form);

			// TODO: move to resource some day
			graphX = 0;
			graphY = 70;
			graphW = 160;
			graphH = 64;

			// streamName was filled by main form
			stream = FileOpen(0, streamName, diaMod->id, CRID, fileModeUpdate, NULL);

//			if(streamReader(0, NULL, 1, offsetof(DiaSampleType,data) + diaMod->sampleDataSize) == 0)
//			{
//			}


			/* Insert SP 06/01/03 - default Hide Unused setting if off */
			FrmSetControlValue(form, FrmGetObjectIndex(form, bankSetupButtonId), 0);
			/* End Insert */

			// panel
			PnlCreatePanel(&panel, panelId, NULL, modPrefs->currentBank, 0, 16, 160, 10 * 4);

			// selector
			SlCreateSelector(&selector,
				diaMod->sensorCount,	/* nItems		*/
				0,						/* pos			*/
				0,						/* selection	*/
				graphX, graphY, graphW-7, graphH);

			// create and paint the graph
			GrfCreateGraph(&graph, graphId, graphX, graphY, graphW, graphH, streamReader, offsetof(DiaSampleType,data) + diaMod->sampleDataSize);

			// TODO: set graph without painting it
//			GrfSetGraph(&graph, 0, 0, 0, graphSensors);
//			GrfSetNumericMask(&graph, captureSensors);
//			GrfSetSensor(&graph, panel.sensorAt[panel.selection]);

			// hilite current bank
			FrmSetControlGroupSelection(form, bankGroupId, bank1ButtonId + panel.currentBank);

			// hilite current mode
			FrmSetControlGroupSelection(form, modeGroupId, reviewButtonId + currentMode);

			WinDrawGrayLine(graphX, graphY - 13, graphX + graphW - 1, graphY - 13);
			WinDrawGrayLine(graphX, graphY - 1, graphX + graphW - 1, graphY - 1);
			WinDrawGrayLine(graphX, graphY + graphH, graphX + graphW - 1, graphY + graphH);

			// paint the form according to current mode
			switchMode(currentMode++);
		}
		return true;

	case frmCloseEvent:
		graphMode = graph._mode;
		currentMode = currentMode & MODE_MASK;
		GrfDestroyGraph(&graph);
		PnlDestroyPanel(&panel);
		SlDestroySelector(&selector);
		{
			Int32 streamSize;
			Err err;

			FileTell(stream, &streamSize, &err);
			FileClose(stream);
			// delete log if empty
			if(streamSize == 0) FileDelete(0, streamName);
//			<= offsetof(DiaSampleType,data) + diaMod->sampleDataSize) FileDelete(0, streamName);
		}
		
		return false;

	case slSelectEvent:
		{
			SelectorEventType *event = (SelectorEventType *)e;

			_setSensorIndex(sensorSelection, event->data.slSelect.selection);
//    		SlEnsureVisible(&selector);
//			selectedSensor = event->data.slSelect.selection;
			_onSelectorSelect();
		}
		return true;

	case slSelectPenUpEvent:
		{
//			SelectorEventType *event = (SelectorEventType *)e;
    		SlEnsureVisible(&selector);
			_onSelectorSelect();
		}
		return true;

	case sclRepeatEvent:
		// user moved scroller - update selector
		if(e->data.sclRepeat.scrollBarID == bankSetupScrollId)
			SlSetPos(&selector, (UInt8)e->data.sclRepeat.newValue);
		break;

	case pnlSelectEvent:
		{
			PanelEventType *event = (PanelEventType *)e;

			_setSensorGraphMask(event->data.pnlSelect.sensorGraphMask);
			_setSensorCaptureMask(event->data.pnlSelect.sensorCaptureMask);
			_setSensorSelection(event->data.pnlSelect.sensorSelection);


//			graphSensors = event->data.pnlSelect.graph;
//			captureSensors = event->data.pnlSelect.capture;
//			modPrefs->panelSelection = event->data.pnlSelect.selection;
		}
		return true;

	case keyDownEvent:
		if(e->data.keyDown.modifiers & autoRepeatKeyMask) break;

		// decrement current bank
		if(e->data.keyDown.chr == pageUpChr) {
//			SndPlaySystemSound(sndClick);
			PnlSetBank(&panel, (panel.currentBank - 1) & 3);
			FrmSetControlGroupSelection(form, bankGroupId, bank1ButtonId + panel.currentBank);
			modPrefs->currentBank = panel.currentBank;
			return true;
		}
		// cincrement current bank
		if(e->data.keyDown.chr == pageDownChr) {
//			SndPlaySystemSound(sndClick);
			PnlSetBank(&panel, (panel.currentBank + 1) & 3);
			FrmSetControlGroupSelection(form, bankGroupId, bank1ButtonId + panel.currentBank);
			modPrefs->currentBank = panel.currentBank;
			return true;
		}
		break;

	case ctlSelectEvent:
		switch(e->data.ctlSelect.controlID) {

		case doneButtonId:
			FrmGotoForm(mainFormId);
			return true;

		case bank1ButtonId:
		case bank2ButtonId:
		case bank3ButtonId:
		case bank4ButtonId:
			PnlSetBank(&panel, (FrmGetControlGroupSelection(form, bankGroupId) - 1) & 3);
			modPrefs->currentBank = panel.currentBank;
			return true;

		case bankSetupButtonId:
			switchMode(currentMode ^ BANK_SETUP_SUBMODE);
//			PnlRefresh(&panel);
			return true;


		case reviewButtonId:
			switchMode(REVIEW_MODE);
			return true;

		case monitorButtonId:
			switchMode(MONITOR_MODE);
			return true;

		}
  		break;

	case frmUpdateEvent:
		{
			RectangleType r = { { graphX,graphY + graphH }, { graphW, 10 } };
			WinEraseRectangle(&r, 0);
		}
		WinDrawGrayLine(graphX, graphY - 13, graphX + graphW - 1, graphY - 13);
		WinDrawGrayLine(graphX, graphY - 1, graphX + graphW - 1, graphY - 1);
		WinDrawGrayLine(graphX, graphY + graphH, graphX + graphW - 1, graphY + graphH);
		PnlRefresh(&panel);
		SlRedraw(&selector);
		GrfAppendSample(&graph, 0);
		break;

	case menuEvent:
		switch (e->data.menu.itemID) {

		case alarmsMenuId:
			FrmPopupForm(alarmsFormId);
			return true;

		case prefsMenuId:
			doPrefsDialog();
//			PnlRefresh(&panel);
//			panel.graph = graphSensors;
//			panel.capture = captureSensors;
//			GrfSetMask(&graph, panel.graph);
//			GrfSetNumericMask(&graph, panel.capture);
			return true;

		case timingMenuId:
			DiaTimingDialog();
			return true;

		case diaPrefsMenuId:
			diaMod->DiaPrefsDialog();
			return true;

		case aboutMenuId:
			doAboutDialog();
			return true;

//		case sensorsMenuId:
//			DiaSensorsHelp();
//			return true;

		case renameLogMenuId:
			{
				FormType *dialog = FrmInitForm(renameLogFormId);
				UInt16 fieldIndex = FrmGetObjectIndex(dialog, newNameFieldId);
				Int16 checkIndex = FrmGetObjectIndex(dialog, backupLogCheckId);
				FieldType *field = FrmGetObjectPtr(dialog, fieldIndex);

				// paste current log name into the field
				FldInsert(field, streamName, StrLen(streamName));
				// select field contents, so it would be deleted if user starts typing
				FldSetSelection(field, 0, StrLen(streamName));
				FrmSetFocus(dialog, fieldIndex);
				FrmSetControlValue(dialog, checkIndex, (streamAttr & dmHdrAttrBackup) ? 1 : 0);

				// show dialog
				if(FrmDoDialog(dialog) == okButtonId) {
					const Char *newName = FldGetTextPtr(field);
					Boolean streamBackup = FrmGetControlValue(dialog, checkIndex);

					// file can't have no name - don't even try
					if(StrLen(newName) <= 0)
						newName = streamName;

					{
						DmOpenRef dbRef = NULL;
						LocalID dbId = 0;
						Int32 size = sizeof(DmOpenRef);
						UInt16 newAttr = dmHdrAttrStream | (streamBackup ? dmHdrAttrBackup : 0);

						(void)(0
						|| FileControl(fileOpGetOpenDbRef, stream, &dbRef, &size)
						|| DmOpenDatabaseInfo(dbRef, &dbId, NULL, NULL, NULL, NULL)
						|| DmSetDatabaseInfo(0, dbId, StrCompare(streamName, newName) ? newName : NULL, &newAttr, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL)
						|| StrCopy(streamName, newName)
						|| (streamAttr = newAttr)
						);
					}
				}

				FrmDeleteForm(dialog);
			}
			return true;

		case newLogMenuId:
			{
				if(FrmAlert(newLogAlertId) == 0) {
				}
			}
			return true;

		case deleteLogMenuId:
			if(FrmAlert(deleteLogAlertId) == 0) {
				// just truncate the log and switch to main
				// form. frmCloseEvent handler will delete
				// the file...
				FileSeek(stream, 0, fileOriginBeginning);
				FileTruncate(stream, 0);
				FrmGotoForm(mainFormId);
			}
			return true;

		case truncateLogMenuId:
			if(FrmAlert(truncateLogAlertId) == 0) {
				FileSeek(stream, 0, fileOriginBeginning);
				FileTruncate(stream, 0);
				GrfUpdateGraph(&graph);
				_setScroller(graph.position);
			}
			return true;

		default:
			/* let the system handle it */
			break;
		}
		break;

	default:
		break;
	}

	return false;
}

/*
 *  m a i n
 */

struct DirEntry {
	Char name[32];
	UInt16 attr;
};

struct DirEntry *dir = NULL;
Int16 dirSize;

#define ITEM_HEIGHT 11
#define ITEM_COUNT 11

void mainDrawItem(UInt16 i, Boolean highlight)
{
	if(i < topVisible + ITEM_HEIGHT) {
		UInt16 strWidth, strLen;
		Char str[32];
		RectangleType clip, rect = { { 0, 18 + i * ITEM_HEIGHT }, { 151, 11 } };

		WinPushDrawState();
		
		if(highlight) {
			WinSetBackColor(UIColorGetTableEntryIndex(UIObjectSelectedFill));
			WinSetTextColor(UIColorGetTableEntryIndex(UIObjectSelectedForeground));
		} else {
			WinSetBackColor(UIColorGetTableEntryIndex(UIObjectFill));
			WinSetTextColor(UIColorGetTableEntryIndex(UIObjectForeground));
		}

		WinEraseRectangle(&rect, 0);
		WinGetClip(&clip);
		WinSetClip(&rect);

		strLen = StrPrintF(str, "%d.", topVisible + i + 1);
		strWidth = FntCharsWidth(str, strLen);
		WinDrawChars(str, strLen, 14 - strWidth, 18 + i * ITEM_HEIGHT);

		if(dir[topVisible + i].attr & dmHdrAttrBackup)
			WinDrawChars("\225", 1, 146, 18 + i * ITEM_HEIGHT);

		rect.extent.x -= 6;
		WinSetClip(&rect);

		WinDrawChars(dir[topVisible + i].name, StrLen(dir[topVisible + i].name), 18, 18 + i * ITEM_HEIGHT);

		WinSetClip(&clip);
		WinPopDrawState();
	}
}

Int16 compare(void *p1, void *p2, Int32 length)
{
	Char* s1 = ((struct DirEntry *)p1)->name;
	Char* s2 = ((struct DirEntry *)p2)->name;
	Int16 r;

	while(*s2) if((r = *s1++ - *s2++)) return r;
	return *s1;

	// couldn't believe it but PalmOS prior to 4.0 have no
	// plain strcmp! StrComapare is doing something obscure...

	/* return StrCompare((Char *)p1, (Char *)p2); */
}

void loadDirectory()
{
	Boolean newSearch = true;
	DmSearchStateType state;
	UInt16 cardNo;
	LocalID dbID;
	Int16 i;
//	Err err;

	if(dir != NULL)
		MemPtrFree(dir);
	dirSize = 16;
	dir = MemPtrNew(dirSize * sizeof(struct DirEntry));

	i = 0;
	while(errNone == DmGetNextDatabaseByTypeCreator(newSearch, &state, diaMod->id/*sysFileTFileStream*/, CRID, false, &cardNo, &dbID)) {
		newSearch = false;
		if(errNone == DmDatabaseInfo(cardNo, dbID,
			dir[i].name, &dir[i].attr /* attributesP */,
			NULL /* versionP */, NULL /* cdDateP */,
			NULL /* modDateP */, NULL /*bckUpDateP */,
			NULL /* modNumP */, NULL /* appInfoIDP */,
			NULL /* sortInfoIDP */,
			NULL /* *typeP */, NULL /* creatorP */))
		{
			if(++i >= dirSize) {
				void *p = MemPtrNew((dirSize + 16) * sizeof(struct DirEntry));
				MemMove(p, dir, dirSize * sizeof(struct DirEntry));
				MemPtrFree(dir);
				dir = p;
				dirSize += 16;
			}
		}
	}

	dirSize = i;
	SysInsertionSort(dir, dirSize, sizeof(struct DirEntry), compare, 32);

	{
		ScrollBarType* bar = FrmGetObjectPtr(FrmGetActiveForm(), FrmGetObjectIndex(FrmGetActiveForm(), mainScrollbarId));

		if(topVisible + ITEM_COUNT > dirSize) topVisible = MAX(0, dirSize - ITEM_COUNT); 


		FrmDrawForm(FrmGetActiveForm());
		for(i = 0; i < MIN(dirSize, ITEM_COUNT); i++)
			mainDrawItem(i, false);
		if(i < ITEM_COUNT) {
			RectangleType rect = { { 0, 18 + i * ITEM_HEIGHT }, { 151, 11*(ITEM_COUNT-i) } };
			WinEraseRectangle(&rect, 0);
		}

		SclSetScrollBar(bar, topVisible, 0, MAX(0, dirSize - ITEM_COUNT), ITEM_COUNT);
	}
}

void loadMods()
{
	UInt16 i;
  	ListPtr list;
  	FormPtr frmP = FrmGetActiveForm();
	static Char *strings[DIA_NMODS];

  	for(i = 0; i < DIA_NMODS; i++)
  		strings[i] = diaModList[i].slug;

	list = FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, mainPopupListId));
	LstSetListChoices(list, strings, DIA_NMODS);
	LstSetHeight(list, DIA_NMODS < 10 ? DIA_NMODS : 10);
	LstSetSelection(list, diaCurrentMod);
	CtlSetLabel(
		FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, mainTriggerId)),
			diaModList[diaCurrentMod].slug
		);
}

void loadPrefs()
{
	UInt16 size = sizeof(Prefs);
	UInt16 chksize = 0;
	UInt32 i;
	Err err = !errNone;

	MemSet(&prefs, size, 0);

	// initialize sensors
//	MemMove(diaSensor, diaMod->sensorDefault, sizeof(diaSensor));

	PrefGetAppPreferences(CRID, 0, NULL, &chksize, false);
	if (chksize == size) err = PrefGetAppPreferences(CRID, 0, &prefs, &size, false);
	if(err != errNone) {
		PrefSetAppPreferences(CRID, 0, 0, NULL, 0, false);

//		for(i = 0; i < SENSOR_COUNT; i++)
//			sensorIndex[i] = i;
//		sensorGraphMask = 0;
//		sensorCaptureMask = 0;
//		sensorSelection = 0;
		currentMode = 0;
		topVisible = 0;
		metricUnit = PREFS_UNIT_METRIC;
		audibleAlarms = true;
		graphMode = GRAPH_NORMAL;
		audibleAlarms = true;
		
//		for(i = 0; i < SENSOR_COUNT; i++) {
//			if(diaSensor[i].capture) captureSensors |= (1L << i);
//			if(diaSensor[i].graph) graphSensors |= (1L << i);
//		}

		for(i = 0; i < DIA_NMODS; i++)
		{
			prefs.mod[i].timing = *diaModList[i].timing = *diaModList[i].timingDefault;
			prefs.mod[i].sensorGraphMask = 0;
			prefs.mod[i].sensorCaptureMask = 0;
			prefs.mod[i].currentBank = 0;
			prefs.mod[i].sensorSelection = 0;

			MemMove(prefs.mod[i].sensorIndex, sensorIndex, SENSOR_COUNT);
		}
	}
	else
	{
		ModPrefs *p = &prefs.mod[diaCurrentMod];

		MemMove(sensorIndex, p->sensorIndex, 32);
//		for(i = 0; i < SENSOR_COUNT; i++) {
//			diaMod->sensor[i].polarity = !!(p->polarity & (1L << i));
//			diaMod->sensor[i].threshold = p->threshold[i];
//			diaMod->sensor[i].sound = p->sound[i];
//		}
		sensorGraphMask = p->sensorGraphMask;
		sensorCaptureMask = p->sensorCaptureMask;
		sensorSelection = p->sensorSelection;
		currentMode = p->currentMode;
		topVisible = p->topVisible;
		audibleAlarms = p->audibleAlarms;
		graphMode = p->graphMode;
		audibleAlarms = p->audibleAlarms;
		diaMod->DiaLoadPrefs(&p->custom);

		*diaMod->timing = p->timing;

		metricUnit = prefs.metricUnit;
	}
}

void savePrefs()
{
//	UInt16 i;
	ModPrefs *p = &prefs.mod[diaCurrentMod];

	diaMod->DiaSavePrefs(&p->custom);

	p->sensorCaptureMask = sensorCaptureMask;
	p->sensorGraphMask = sensorGraphMask;
	p->sensorSelection = sensorSelection;
	p->currentMode = currentMode & MODE_MASK;
	p->topVisible = topVisible;
	p->graphMode = graphMode;
	p->audibleAlarms = audibleAlarms;
	p->polarity = 0;

	MemMove(p->sensorIndex, sensorIndex, 32);
//	for(i = 0; i < SENSOR_COUNT; i++) {
//		p->threshold[i] = diaMod->sensor[i].threshold;
//		if(diaMod->sensor[i].polarity) p->polarity |= (1L << i);
//		p->sound[i] = diaMod->sensor[i].sound;
//		p->sensorIndex[i] = sensorIndex[i];
//	}

	p->timing = *diaMod->timing;

	prefs.metricUnit = metricUnit;
	prefs.currentMod = diaCurrentMod;

	PrefSetAppPreferences(CRID, 0, 0, &prefs, sizeof(prefs), false);
}

Boolean mainFormHandleEvent(EventType *e)
{
	FormType *form = FrmGetActiveForm();
	static Int16 hilitedItem;
	Int16 i;
	Err err;

	if(SysHandleEvent(e) || MenuHandleEvent(NULL, e, &err)) return true;
	
	switch(e->eType) {

	case frmOpenEvent:

		diaMod->DiaStop();
		FrmDrawForm(form);
		hilitedItem = -1;

		loadMods();

		loadDirectory();

		return true;

	case frmCloseEvent:
		MemPtrFree(dir);
		dir = NULL;
		return false;

	case sclRepeatEvent:
		topVisible = e->data.sclRepeat.newValue;
		
		for(i = 0; i < MIN(dirSize, ITEM_COUNT); i++)
			mainDrawItem(i, false);

		return false;
	
	case penDownEvent:
		if(e->screenX < 150 && e->screenY >= 18 && e->screenY <= 18 + ITEM_COUNT * ITEM_HEIGHT) {
			i = (e->screenY - 18) / ITEM_HEIGHT;
			if(i >= 0 && i < dirSize) {
				mainDrawItem(i, true);
				hilitedItem = i;
	                	return true;
			}
		}
		return false;

	case penMoveEvent:
		i = (e->screenY - 18) / ITEM_HEIGHT;
		if(hilitedItem >= 0 && i != hilitedItem) {
			mainDrawItem(hilitedItem, false);
			hilitedItem = -1;
                	return true;
		}
		return false;

	case penUpEvent:
		i = (e->screenY - 18) / ITEM_HEIGHT;
		if(hilitedItem >= 0 && i == hilitedItem) {
			StrCopy(streamName, dir[topVisible + i].name);
			streamAttr = dir[topVisible + i].attr;
			StrPrintF(editFormTitle, "%d/%d", i + 1 + topVisible, dirSize);
			FrmGotoForm(editFormId);
			return true;
		} else if(hilitedItem >= 0) {
			mainDrawItem(hilitedItem, false);
			hilitedItem = -1;
		}
		
		return false;

	case ctlSelectEvent:
/*		if(e->data.ctlSelect.controlID == testButtonId) {
			Err err;
			SrmOpenConfigType cfg;
			UInt16 port;



			dbgInfo("starting");

			cfg.baud = 0;
			cfg.function = serFncUndefined;
			cfg.drvrDataP = NULL;
			cfg.drvrDataSize = 0;

			err = SrmExtOpen(serPortCradleUSBPort, &cfg, sizeof(cfg), &port);
			if(err != errNone)
			{
				dbgInfo("SrmExtOpen() err %x", err);
				return false;
			}

			dbgInfo("port open");

			SrmClose(port);

			return false;
		}*/
		if(e->data.ctlSelect.controlID == newButtonId) {
			DateTimeType date;

			TimSecondsToDateTime(TimGetSeconds(), &date);
			StrPrintF(streamName, "%02d-%02d-%02d %02dh%02dm", date.year % 100, date.month, date.day, date.hour, date.minute);
			streamAttr = dmHdrAttrStream;

			if(DmFindDatabase(0, streamName) != 0)
			{
				Char name[32];
				Char suffix[8];
				UInt16 counter = 0;

				do
				{
					StrCopy(name, streamName);
					StrPrintF(suffix, " %03u", ++counter);
					StrCat(name, suffix);
				}
				while(DmFindDatabase(0, name) != 0);

				StrCopy(streamName, name);
			}

			StrPrintF(editFormTitle, "%d/%d", dirSize+1, dirSize+1);
			FrmGotoForm(editFormId);
			return true;
		}
		return false;

	case popSelectEvent:
		switch(e->data.popSelect.controlID)
		{
		case mainTriggerId:
			if(diaCurrentMod == e->data.popSelect.selection)
				break;

			/* switch to selected mod */

			// save mod prefs
			savePrefs();

			// switch
			diaCurrentMod = e->data.popSelect.selection;
			diaMod = &diaModList[diaCurrentMod];
			modPrefs = &prefs.mod[diaCurrentMod];

			// load mod prefs
			loadPrefs();

			// load directory
			loadDirectory();

			break;
		}
		break;

	case menuEvent:
		switch (e->data.menu.itemID) {

		case alarmsMenuId:
			FrmPopupForm(alarmsFormId);
			return true;

		case aboutMenuId:
			doAboutDialog();
			return true;

//		case sensorsMenuId:
//			DiaSensorsHelp();
//			return true;

		case testMenuId:
			FrmGotoForm(testFormId);
			return true;

		case debugMenuId:
			FrmGotoForm(debugFormId);
			return true;

		case prefsMenuId:
			doPrefsDialog();
			return true;

		case timingMenuId:
			DiaTimingDialog();
			return true;

		case diaPrefsMenuId:
			diaMod->DiaPrefsDialog();
			return true;

		default:
			break;

		}
	default:
		break;
	}

	return false;
}

/*
 *
 */

Boolean appHandleEvent(EventType *e)
{
	if(e->eType == frmLoadEvent) {
		// Load the form resource.
		UInt16 formID = e->data.frmLoad.formID;
		FormPtr form = FrmInitForm(formID);
		FrmSetActiveForm(form);

		switch(formID) {

		case mainFormId:
			FrmSetEventHandler(form, mainFormHandleEvent);
			break;

		case editFormId:
			FrmSetEventHandler(form, editFormHandleEvent);
			break;

		case testFormId:
			FrmSetEventHandler(form, testFormHandleEvent);
			break;

		case debugFormId:
			FrmSetEventHandler(form, debugFormHandleEvent);
			break;

		case alarmsFormId:
			FrmSetEventHandler(form, alarmsFormHandleEvent);
			break;

		default:
			ErrFatalDisplayIf(1, "Unknown form!");
			break;
		}
	}
	return false;
}

UInt32 PilotMain(UInt16 cmd, void *pbp, UInt16 flags __attribute__ ((unused)))
{
	switch(cmd) {
	case sysAppLaunchCmdNormalLaunch:
		{
			EventType event;
			Boolean screenEnableColor;
			UInt32 version;

			// check if PalmOS version is 3.5 or newer
			if(errNone != FtrGet(sysFtrCreator, sysFtrNumROMVersion, &version) || version < 0x03503000) {
				FrmAlert(versionAlertId);
				return 0;
			}

			colorMode = false;
			screenDensity = 1;

			if(errNone == WinScreenMode(winScreenModeGet,
			   &screenWidth, &screenHeight, &screenDepth,
			   &screenEnableColor)) {
				if(screenDepth >= 8) colorMode = true;
			}

			if(errNone == FtrGet(sysFtrCreator, sysFtrNumWinVersion, &version) && version >= 4) {
				UInt32 attr;
				WinScreenGetAttribute(winScreenDensity, &attr);

				switch(attr) {
				case kDensityDouble:
					screenDensity = 2;
					break;
				case kDensityTriple:
					screenDensity = 3;
					break;
				case kDensityQuadruple:
					screenDensity = 4;
					break;
				}
			}

#ifdef HANDSPRING_SUPPORT
			// If on Handspring device, disable the keyboard
			// thread before opening the serial library.
			if(!FtrGet('hsEx', 0, &version))
				HsExtKeyboardEnable(false);
#endif


			ticksPerSecond = SysTicksPerSecond();

			diaMod = &diaModList[diaCurrentMod];
			modPrefs = &prefs.mod[diaCurrentMod];
			loadPrefs();

			if(prefs.currentMod != diaCurrentMod)
			{
				diaCurrentMod = prefs.currentMod;
				diaMod = &diaModList[diaCurrentMod];
				modPrefs = &prefs.mod[diaCurrentMod];
				loadPrefs();
			}

			FrmGotoForm(mainFormId);

			bigFontH = DmGetResource('NFNT', bigFontId);
			bigFontP = MemHandleLock(bigFontH);
			FntDefineFont(fntAppFontCustomBase, bigFontP);

			talkingECU = false;

			do {
				/* Modified SP 06/03/03 - Wait up to 1 second for events */
				/* reduced timeout to 1/50 sec, because null events are
				   used by graph widget to scroll diplay when pen
				   is dragged to the edge of the screen -/dmitry */
				EvtGetEvent(&event, talkingECU ? 0 : ticksPerSecond / 50);
				/* End Modified */
				(void) (
				        appHandleEvent(&event)
				     || FrmDispatchEvent(&event)
				);
			} while(event.eType != appStopEvent);

			FrmCloseAllForms();
			diaMod->DiaStop();	// just in case
			savePrefs();

			MemHandleUnlock(bigFontH);
			DmReleaseResource(bigFontH);
		}
		break;

	case sysAppLaunchCmdSystemReset: 
		break;

	case sysAppLaunchCmdAlarmTriggered:
		{
			SysAlarmTriggeredParamType *param = pbp;
			param->purgeAlarm = true;
		}
		break;
	}

	return 0;
}

