/* 
 *  $Id: format.c,v 1.1 2003/08/13 13:40:27 kogan Exp $
 *
 *  Copyright (c) 2003, Sergey Kogan
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#include <PalmOS.h>
#include "../config.h"
#include "../dia.h"

/*** protocol specific formatting functions implementation ***/

#define DEG "\260"	// degree symbol

// E
DIA_DEFINE_FMT(mp72SPD)
{
	return StrPrintF(fmt, "%ukm/h", UINT16(*data));
}

// E*100/256
DIA_DEFINE_FMT(mp72TPS)
{
	return StrPrintF(fmt, "%u%%", (UINT16(*data)*100)/256);
}

// E * 40
DIA_DEFINE_FMT(mp72RPM)
{
	UInt16 rpm = UINT16(*data)*40;
	return StrPrintF(fmt, "%drpm", rpm);
}

/*
// E - 40
DIA_DEFINE_FMT(mp72COOL)
{
	return StrPrintF(fmt, "%d" DEG "C", INT16(*data) - 40);
}

// E/256
DIA_DEFINE_FMT(mp72COCR)
{
	UInt32 v = UINT32(*data) * 257L;
	return StrPrintF(fmt, "%ld.%03ld", v >> 16, (v & 0xffff) * 1000L >> 16);

}

// E * 10
DIA_DEFINE_FMT(mp72XRPM)
{
	return StrPrintF(fmt, "%drpm", UINT16(*data)*10);
}

// 5.2 + E * 0.05
DIA_DEFINE_FMT(mp72BATT)
{
	UInt32 v = 340787L + 3277L * UINT32(*data);
	return StrPrintF(fmt, "%ld.%ldV", v >> 16, (v & 0xffff) * 10 >> 16);
}

// E / 10
DIA_DEFINE_FMT(mp72MAFS)
{
	UInt16 val = UINT16(*data)<<8|UINT16(*(data+1));
//	UINT16(data[0]) | UINT16(data[1]) << 8;
	return StrPrintF(fmt, "%u.%ukg/h", val / 10, val % 10);
}

// E / 6
DIA_DEFINE_FMT(mp72CAFS)
{
	UInt32 v = UINT32(UINT16(*data)<<8|UINT16(*(data+1))) * 10992L;
	return StrPrintF(fmt, "%ld.%01ldmg/c", v >> 16, (v & 0xffff) * 10L >> 16);
}

// E / 2
DIA_DEFINE_FMT(mp72TIMA)
{
	UInt16 val = INT16(*data);
	return StrPrintF(fmt, "%d.%d" DEG, val/2, 5*(val%2));
}

// (E + 128)/256
DIA_DEFINE_FMT(mp72INJC)
{
	UInt32 v = (UINT32(*data) + 128L) * 257L;
	return StrPrintF(fmt, "%ld.%03ld", v >> 16, (v & 0xffff) * 1000L >> 16);

}

// E/125
DIA_DEFINE_FMT(mp72INJP)
{
	UInt32 v =
		UINT32(UINT16(*data)<<8|UINT16(*(data+1)))
//	(UINT32(data[0]) | UINT32(data[1]) << 8)
	* 524L;
	return StrPrintF(fmt, "%ld.%03ldms", v >> 16, (v & 0xffff) * 1000L >> 16);

}

// E/128
DIA_DEFINE_FMT(mp72FUEL)
{
	UInt32 v = //(UINT32(data[0]) | UINT32(data[1]) << 8);
		UINT32(UINT16(*data)<<8|UINT16(*(data+1)));
	if(v == 0xFFFF) {
		StrCopy(fmt, "inf");
		return 3;
	}
	v *= 512L;
	return StrPrintF(fmt, "%ld.%ldl", v >> 16, (v & 0xffff) * 10L >> 16);
}

// E/50
DIA_DEFINE_FMT(mp72HFUE)
{
	UInt32 v = 
		UINT32(UINT16(*data)<<8|UINT16(*(data+1)))
		* 1311L;
	return StrPrintF(fmt, "%ld.%ldl/h", v >> 16, (v & 0xffff) * 10L >> 16);
}


DIA_DEFINE_SCALE(mp72RPM)
{
	return vsize - 1 - UINT8((UINT16(*data)*vsize)/200);
}

DIA_DEFINE_SCALE(mp72TPS)
{
	return vsize - 1 - UINT8((UINT16(*data)*vsize)/110);
}

DIA_DEFINE_SCALE(mp72MAFS)
{
	return vsize - 1 - UINT8(
		UINT32(UINT16(*data)<<8|UINT16(*(data+1)))
//	UINT32(UINT16(data[addr]) | UINT16(data[addr+1]) << 8)
	*UINT32(vsize)/1600L);
}

DIA_DEFINE_SCALE(mp72CAFS)
{
	return vsize - 1 - UINT8(
		UINT32(UINT16(*data)<<8|UINT16(*(data+1)))
//	UINT32(UINT16(data[addr]) | UINT16(data[addr+1]) << 8)
	*UINT32(vsize)/2200L);
}

DIA_DEFINE_SCALE(mp72INJP)
{
	return vsize - 1 - UINT8(
		UINT32(UINT16(*data)<<8|UINT16(*(data+1)))
//	UINT32(UINT16(data[addr]) | UINT16(data[addr+1]) << 8)
	*UINT32(vsize)/1000L);
}

DIA_DEFINE_SCALE(mp72FUEL)
{
	UInt32 v = UINT32(UINT16(*data)<<8|UINT16(*(data+1)));//UINT32(UINT16(data[addr]) | UINT16(data[addr+1]) << 8);
	if(v == 0xFFFF) return 0;
	return vsize - 1 - UINT8(v*UINT32(vsize)/65536L);
}

DIA_DEFINE_SCALE(mp72HFUE)
{
	return vsize - 1 - UINT8(UINT32(UINT16(*data)<<8|UINT16(*(data+1)))
//	UINT32(UINT16(data[addr]) | UINT16(data[addr+1]) << 8)
	*UINT32(vsize)/1000L);
}

*/