/* 
 *  $Id: mp72.c,v 1.3 2003/11/26 15:54:15 kogan Exp $
 *
 *  Copyright (c) 2003, Sergey Kogan
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#include <PalmOS.h>
#include "../config.h"
#include "../dia.h"
#include "mp7.h"
#include "mp72.h"
#include "format.h"


/* sensors */
const DiaSensorType mp72SensorDefault[] = {
//
// E - exists, Sn - sound, P - polarity,
// Thr - threshold, Clr - color
//
//addr  mask         slug    format		scale		compare	E  Sn P Thr color						value        desc
{ 0x00, 0x00000000L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

{ 0x11, 0x00000001L, "RPM",  f_mp72RPM,		s_DECBYTE,	c_BYTE,	1, 0, 0, 0, {0, 0xFF, 0x00, 0x00},	"?\0      ", "Engine speed" },
{ 0x0F, 0x00000001L, "TPS",  f_mp72TPS,		s_DECBYTE,	c_BYTE,	1, 0, 0, 0, {0, 0xFF, 0x00, 0xFF},	"?\0      ", "Throttle position" },
{ 0x10, 0x00000001L, "VS",   f_mp72SPD,		s_DECBYTE,	c_BYTE,	1, 0, 0, 0, {0, 0x00, 0x00, 0xA0},	"?\0      ", "Vehicle speed" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

/* AD channels */
{ 0x26, 0x00000002L, "BATv", f_DEC,		s_DECBYTE,	c_BYTE,	1, 0, 0, 0, {0, 0x60, 0xA0, 0x60},	"?\0      ", "Battery voltage AD code" },
{ 0x27, 0x00000004L, "ECTv", f_DEC,		s_DECBYTE,	c_BYTE,	1, 0, 0, 0, {0, 0x00, 0xA0, 0x00},	"?\0      ", "ECT sensor AD code" },
{ 0x28, 0x00000008L, "MAFv", f_DEC,		s_DECBYTE,	c_BYTE,	1, 0, 0, 0, {0, 0x00, 0x00, 0xA0},	"?\0      ", "Mass air flow sensor AD code" },
{ 0x29, 0x00000010L, "TPSv", f_DEC,		s_DECBYTE,	c_BYTE,	1, 0, 0, 0, {0, 0xA0, 0x00, 0xA0},	"?\0      ", "Throttle position sensor AD code" },
{ 0x2A, 0x00000020L, "LSUv", f_DEC,		s_DECBYTE,	c_BYTE,	1, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "Lambda sensor upstream AD code" },
{ 0x2B, 0x00000040L, "KNKv", f_DEC,		s_DECBYTE,	c_BYTE,	1, 0, 0, 0, {0, 0x30, 0x30, 0x30},	"?\0      ", "Knock control circuit AD code" },
{ 0x2C, 0x00000080L, "LH2v", f_DEC,		s_DECBYTE,	c_BYTE,	1, 0, 0, 0, {0, 0x60, 0x60, 0x60},	"?\0      ", "Lambda sensor heating 2 AD code" },
{ 0x2D, 0x00000100L, "LH1v", f_DEC,		s_DECBYTE,	c_BYTE,	1, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "Lambda sensor heating 1 AD code" },

{ 0x2E, 0x00000200L, "LSDv", f_DEC,		s_DECBYTE,	c_BYTE,	1, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "Lambda sensor downstream AD code" },
/* AD channels done */
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,	0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
};

/*** internal types ***/

/* ctl handlers */
DIA_DEFINE_CTL(Mp72Poll		) MP72_SECT;

const DiaCtlType mp72ctl[] = {
NULL,			/* RESERVED_0	*/
ctl_Mp7EcuReset,	/* ECU_RESET	*/
ctl_Mp7ReadIdent,	/* READ_IDENT	*/
ctl_Mp7ClearErrors,	/* CLEAR_ERRORS	*/
ctl_Mp7ReadErrors,	/* READ_ERRORS	*/
ctl_Mp7Connect,		/* CONNECT	*/
ctl_Mp72Poll,		/* POLL		*/
ctl_Mp7Disconnect,	/* DISCONNECT	*/
ctl_Mp7Ping,		/* PING		*/
ctl_Mp7Reconnect,	/* RECONNECT	*/
ctl_Mp7SetTiming,	/* SETTIMING	*/
};

const DiaTimingType	mp72TimingDefault = {
	0,		/* Min P1 */
	20,		/* Max P1 */
	25,		/* Min P2 */
	200,		/* Max P2 */
	70,		/* Min P3 */
	5000,		/* Max P3 */
	5,		/* Min P4 */
	20		/* Max P4 */
};
DiaTimingType		mp72Timing;

UInt8 mp72step2sensor[] = { 0x00, 
	0x00,
	0x02,
	0x03,
	0x04,
	0x05,
	0x07,
	0x08,
	0x09,
	0x12
};

DIA_DEFINE_CTL(Mp72Poll)
{
	UInt8 req1[] = { 0x21, 0x01 };
	UInt8 req2[] = { 0x23, 0x04, 0x00, 0x00, 0x01 };
	UInt8 step;

	// _mp7Req.recv array structure:
	// 0..37	- afterSalesServiceRecordLocalId data
	// 38..46	- ad channels
	// ----------------
	// sample size = 48
	//
	// 191		- next request id
	// 192..255	- reserved for ReadMemoryByAddress response
	//
	// mask bits:
	// 0x00000001	- afterSalesServiceRecordLocalId
	// 0x00000002	- AD battery voltage
	// 0x00000004	- AD engine temperature sensor
	// 0x00000008	- AD air mass sensor
	// 0x00000010	- AD throttle position
	// 0x00000020	- AD lambda sensor upstream cat.
	// 0x00000040	- AD knock control circuit CC195
	// 0x00000080	- AD ISHZ2 (lambda sensor heating)
	// 0x00000100	- AD ISHZ1 (lambda sensor heating)
	// 0x00000200	- AD lambda sensor downstream cat.

	step = _mp7Req.recv[191];
	
	// do reply relocation
	if(op != DIA_SEND && _mp7Req.recv_to != 0)
		_mp7Req.recv[38+step-1] = _mp7Req.recv[193];
	
	// search for next step
	if(op == DIA_SEND) // first step
		step = 0;
	while(step < 10 && diaMod->mask & (1 << step))
		step++;
	_mp7Req.recv[191] = step + 1;

	// request pending?
	if(step < 10) {
		if(step == 0)
			return _Mp7SendAsync(req1, 2, 0);
		else {
			req2[3] = mp72step2sensor[step];
			return _Mp7SendAsync(req2, 5, 192);
		}
	} else {
		_mp7SampleP = (DiaSampleType *)&_mp7Req.sample;
		_mp7SampleP->time = TimGetSeconds();
		_mp7SampleP->sensorPresent = diaMod->mask;
		_mp7SampleReady = true;
		return DIA_RESULT;
	}
}
