/* 
 *  $Id: mp73.c,v 1.4 2003/11/27 13:58:58 kogan Exp $
 *
 *  Copyright (c) 2003, Sergey Kogan
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#include <PalmOS.h>
#include "../config.h"
#include "../dia.h"
#include "mp73.h"
#include "format.h"
#include "../mp72/mp7.h"
#include "../mp72/mp72.h"
#include "../mp72/format.h"


/*** constants ***/

/* sensors */
const DiaSensorType mp73SensorDefault[] = {
//
// E - exists, Sn - sound, P - polarity,
// Thr - threshold, Clr - color
//
//addr  mask        slug    format		scale		compare		E  Sn P Thr color			value        desc
{ 0x00, 0x00000000L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

{ 0x11, 0x00000001L, "RPM",  f_mp72RPM,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xFF, 0x00, 0x00},	"?\0      ", "Engine speed" },
{ 0x0F, 0x00000001L, "TPS",  f_mp72TPS,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xFF, 0x00, 0xFF},	"?\0      ", "Throttle position" },
{ 0x10, 0x00000001L, "VS",   f_mp72SPD,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0x00, 0xA0},	"?\0      ", "Vehicle speed" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

/* AD channels */
{ 0x59, 0x00000002L, "BATv", f_DEC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x60, 0xA0, 0x60},	"?\0      ", "Battery voltage AD code" },
{ 0x5A, 0x00000004L, "IATv", f_DEC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x60, 0xA0, 0x60},	"?\0      ", "Inlet air temperature AD code" },
{ 0x5B, 0x00000008L, "ECTv", f_DEC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0xA0, 0x00},	"?\0      ", "ECT sensor AD code" },
{ 0x5C, 0x00000010L, "MAFv", f_DEC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0x00, 0xA0},	"?\0      ", "Mass air flow sensor AD code" },
{ 0x5D, 0x00000020L, "TPSv", f_DEC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xA0, 0x00, 0xA0},	"?\0      ", "Throttle position sensor AD code" },
{ 0x5E, 0x00000040L, "LSUv", f_DEC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "Lambda sensor upstream AD code" },
{ 0x5F, 0x00000080L, "LSDv", f_DEC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "Lambda sensor downstream AD code" },
{ 0x60, 0x00000100L, "KNKv", f_DEC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x30, 0x30, 0x30},	"?\0      ", "Knock control circuit AD code" },

{ 0x61, 0x00000200L, "LHUv", f_DEC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x60, 0x60, 0x60},	"?\0      ", "L-sensor heating upstream AD code" },
{ 0x62, 0x00000400L, "ACLv", f_DEC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x60, 0x60, 0x60},	"?\0      ", "Accel. sensor / rough road AD code" },
{ 0x63, 0x00000800L, "LHDv", f_DEC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "L-sensor heating downstream AD code" },
/* AD channels done */
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,		s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

};

const DiaTimingType	mp73TimingDefault = {
	0,	/* Min P1 */
	20,	/* Max P1 */
	25,	/* Min P2 */
	200,	/* Max P2 */
	70,	/* Min P3 */
	5000,	/* Max P3 */
	5,	/* Min P4 */
	20	/* Max P4 */
};
DiaTimingType		mp73Timing;

/* ctl handlers */
DIA_DEFINE_CTL(Mp73Poll		) MP73_SECT;

const DiaCtlType mp73ctl[] = {
NULL,			/* RESERVED_0		*/
ctl_Mp7EcuReset,	/* ECU_RESET		*/
ctl_Mp7ReadIdent,	/* READ_IDENT		*/
ctl_Mp7ClearErrors,	/* CLEAR_ERRORS		*/
ctl_Mp7ReadErrors,	/* READ_ERRORS		*/
ctl_Mp7Connect,		/* CONNECT		*/
ctl_Mp73Poll,		/* POLL			*/
ctl_Mp7Disconnect,	/* DISCONNECT		*/
ctl_Mp7Ping,		/* PING			*/
ctl_Mp7Reconnect,	/* RECONNECT		*/
ctl_Mp7SetTiming,	/* SETTIMING		*/
};

UInt8 mp73step2sensor[] = { 0x00, 
	0x00,
	0x01,
	0x02,
	0x03,
	0x04,
	0x05,
	0x06,
	0x07,
	0x09,
	0x0D,
	0x12
};

DIA_DEFINE_CTL(Mp73Poll)
{
	UInt8 req1[] = { 0x21, 0x01 };
	UInt8 req2[] = { 0x23, 0x04, 0x00, 0x00, 0x01 };
	UInt8 step;

	// _mp7Req.recv array structure:
	// 0..88	- afterSalesServiceRecordLocalId data
	// 89..99	- ad channels
	// ----------------
	// sample size = 100
	//
	// 191		- next request id
	// 192..255	- reserved for ReadMemoryByAddress response
	//
	// mask bits:
	// 0x00000001	- afterSalesServiceRecordLocalId
	// 0x00000002	- AD battery voltage
	// 0x00000004	- AD inlet air temperature sensor
	// 0x00000008	- AD engine temperature sensor
	// 0x00000010	- AD air mass sensor
	// 0x00000020	- AD throttle position
	// 0x00000040	- AD lambda sensor upstream catalyst
	// 0x00000080	- AD lambda sensor downstream catalyst
	// 0x00000100	- AD integral of knock control circuit CC195
	// 0x00000200	- AD lambda sensor heating upstream catalyst / shunt
	// 0x00000400	- AD acceleration sensor / rough road detection
	// 0x00000800	- AD lambda sensor heating downstream catalyst / shunt

	step = _mp7Req.recv[191];
	
	// do reply relocation
	if(op != DIA_SEND && _mp7Req.recv_to != 0)
		_mp7Req.recv[89+step-1] = _mp7Req.recv[193];
	
	// search for next step
	if(op == DIA_SEND) // first step
		step = 0;
	while(step < 12 && diaMod->mask & (1 << step))
		step++;
	_mp7Req.recv[191] = step + 1;

	// request pending?
	if(step < 12) {
		if(step == 0)
			return _Mp7SendAsync(req1, 2, 0);
		else {
			req2[3] = mp73step2sensor[step];
			return _Mp7SendAsync(req2, 5, 192);
		}
	} else {
		_mp7SampleP = (DiaSampleType *)&_mp7Req.sample;
		_mp7SampleP->time = TimGetSeconds();
		_mp7SampleP->sensorPresent = diaMod->mask;
		_mp7SampleReady = true;
		return DIA_RESULT;
	}
}
