/*
 *  $Id: panel.c,v 1.11 2003/11/26 15:54:14 kogan Exp $
 *
 *  Copyright (c) 2003, Dmitry Yurtaev <dm1try@umail.ru>
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#include <PalmOS.h>
#include <Font.h>
#include "dia.h"
#include "panel.h"
#include "mmcd.h"

#define W1 31
#define W2 47

extern Boolean colorMode;

void _PnlDrawButton(PanelType *panel, UInt16 row, UInt16 column, UInt16 field, Boolean blit)
{
	RectangleType rect = { { column * 81 + field * (W1 + 1), row * 10 }, { field ? W2 : W1, 9 } };
	RectangleType fullRect = { { 0, 0 }, panel->bounds.extent };
	WinHandle oldWin;
	FontID oldFontId;
	UInt16 labelWidth, labelLen;
	UInt16 index = panel->currentBank * 8 + row * 2 + column;
	UInt8 sensor = sensorIndex[index];

	// paint to offscreen
	oldWin = WinSetDrawWindow(panel->_offscreen);
	WinPushDrawState();

	// if sensor doesn't exist just erase its button
	if((sensor >= diaMod->sensorCount ||
		(!panel->sensor[sensor].exists)) &&
		!(panel->_mode == PANEL_SINGLESELECT_SLUG && field == 0)) {
		WinSetBackColor(UIColorGetTableEntryIndex(UIObjectFill));
		WinEraseRectangle(&rect, 0);
	} else {
		Boolean active;
		const Char *label;

		if(field == 0) {	//SW
			if(panel->_mode == PANEL_SINGLESELECT_SLUG)
				active = index == sensorSelection;
			else
				active = (sensorCaptureMask & diaMod->sensor[sensor].mask) != 0;
			label = panel->sensor[sensor].exists ? 
				panel->sensor[sensor].slug : 
				(panel->_mode == PANEL_SINGLESELECT_SLUG ? "-" : "");
		} else {
			if(panel->_mode == PANEL_MULTISELECT)
				active = (sensorGraphMask & (1L << index)) != 0;
			else if(panel->_mode == PANEL_SINGLESELECT_VAL)
				active = index == sensorSelection;
			else
				active = false;
			label = panel->sensor[sensor].value;
		}

		if(active) {
			if(colorMode && field == 1 && panel->_mode == PANEL_MULTISELECT)
				WinSetBackColor(sensorColor[index]);//WinRGBToIndex(&(panel->sensor[sensor].color)));
			else
				WinSetBackColor(UIColorGetTableEntryIndex(UIObjectSelectedFill));
			WinSetForeColor(UIColorGetTableEntryIndex(UIObjectSelectedForeground));
			WinSetTextColor(UIColorGetTableEntryIndex(UIObjectSelectedForeground));
		} else {
			WinSetBackColor(UIColorGetTableEntryIndex(UIObjectFill));
			WinSetForeColor(UIColorGetTableEntryIndex(UIObjectForeground));
			WinSetTextColor(UIColorGetTableEntryIndex(UIObjectForeground));
		}


		WinEraseRectangle(&rect, 0);

		if((labelLen = StrLen(label))) {
			labelWidth = FntCharsWidth(label, labelLen) - 2;

			oldFontId = FntSetFont(stdFont);
			WinSetClip(&rect);
			WinDrawChars(label, labelLen,
				rect.topLeft.x + (rect.extent.x - labelWidth) / 2,
				rect.topLeft.y + ((Int16)rect.extent.y - FntCharHeight()) / 2
			);
			WinSetClip(&fullRect);
			FntSetFont(oldFontId);
		}
	}

	WinPopDrawState();
	WinSetDrawWindow(oldWin);

	if(blit && panel->visible)
		WinCopyRectangle(panel->_offscreen, oldWin, &rect, panel->bounds.topLeft.x + rect.topLeft.x, panel->bounds.topLeft.y + rect.topLeft.y, winPaint);
}

void _PnlDrawPanel(PanelType *panel)
{
	UInt16 row, column, field;
	RectangleType rect = { { 0, 0 }, panel->bounds.extent };

	for(row = 0; row < panel->rows; row++)
		for(column = 0; column < 2; column++)
			for(field = 0; field < 2; field++)
				_PnlDrawButton(panel, row, column, field, false);

	if(panel->visible)
		WinCopyRectangle(panel->_offscreen, WinGetDisplayWindow(), &rect, panel->bounds.topLeft.x, panel->bounds.topLeft.y, winPaint);
}

void PnlUpdate(PanelType *panel, const DiaSampleType* sample)
{
	UInt16 i, updated = 0;
	UInt8 index, sensor;

	if(sample == NULL) {
		for(i = 0; i < panel->rows * 2; i++) {
			index = panel->currentBank*8 + i;
			sensor = sensorIndex[index];
			if(sensor >= diaMod->sensorCount) continue;
			if((panel->_sample.sensorPresent & diaMod->sensor[sensor].mask) != 0) {
				panel->sensor[sensor].format(
					&panel->sensor[sensor],
					&panel->_sample.data[panel->sensor[sensor].addr],
					panel->sensor[sensor].value);
			} else {
				panel->sensor[sensor].value[0] = '-';
				panel->sensor[sensor].value[1] = 0;
			}
			updated |= 1L << i;
		}
	} else {
		for(i = 0; i < panel->rows * 2; i++) {
			index = panel->currentBank*8 + i;
			sensor = sensorIndex[index];
			if(sensor >= diaMod->sensorCount) continue;

			if((sample->sensorPresent & diaMod->sensor[sensor].mask) != 0) {
				panel->sensor[sensor].format(
					&panel->sensor[sensor],
					(UInt8*)&sample->data[panel->sensor[sensor].addr],
					panel->sensor[sensor].value);
			} else {
				panel->sensor[sensor].value[0] = '-';
				panel->sensor[sensor].value[1] = 0;
			}
			updated |= 1L << i;
		}
		panel->_sample = *sample;

		if(updated) {
			for(i = 0; i < panel->rows * 2; i++) {
				if(updated & (1L << i))
					_PnlDrawButton(panel, i / 2, i & 1, 1, false);
			}

			if(panel->visible) {
				RectangleType r0 = { { W1 + 1, 0 }, { W2, panel->bounds.extent.y } };
				RectangleType r1 = { { panel->bounds.extent.x - W2, 0 }, { W2, panel->bounds.extent.y } };

				WinCopyRectangle(panel->_offscreen, WinGetActiveWindow(), &r0, r0.topLeft.x + panel->bounds.topLeft.x, r0.topLeft.y + panel->bounds.topLeft.y, winPaint);
				WinCopyRectangle(panel->_offscreen, WinGetActiveWindow(), &r1, r1.topLeft.x + panel->bounds.topLeft.x, r1.topLeft.y + panel->bounds.topLeft.y, winPaint);
			}
		}
	}
}

void PnlSetBank(PanelType *panel, UInt8 bank)
{
	panel->currentBank = bank;
	PnlRefresh(panel);
}

void PnlRefresh(PanelType *panel)
{
	PnlUpdate(panel, NULL);
	_PnlDrawPanel(panel);
}


void PnlCreatePanel(PanelType *panel, UInt16 id, DiaSensorType *sensor, UInt8 currentBank, Coord x, Coord y, Coord width, Coord height)
{
	Err err;
//	UInt16 i;
	WinHandle oldWin;

	sensor = diaMod->sensor;	// TODO

	panel->bounds.topLeft.x = x;
	panel->bounds.topLeft.y = y;
	panel->bounds.extent.x = width;
	panel->bounds.extent.y = height;

	panel->rows = height / 10;
	panel->visible = 1;

//	panel->graph = 0;
//	panel->capture = 0;

	panel->_mode = PANEL_MULTISELECT;

//	for(i = 0; i < 32; i++) {
//		sensorIndex[i] = i;
//		panel->graph |= (UInt32)sensor[i].graph << i;
//		panel->capture |= (UInt32)sensor[i].capture << i;
//	}
	panel->_sample.sensorPresent = 0;

	panel->sensor = sensor;
	panel->_offscreen = WinCreateOffscreenWindow(width, height, nativeFormat, &err);

	oldWin = WinSetDrawWindow(panel->_offscreen);
	WinPushDrawState();
	WinSetBackColor(UIColorGetTableEntryIndex(UIObjectFill));
	WinEraseWindow();
	WinPopDrawState();
	WinSetDrawWindow(oldWin);

	PnlSetBank(panel, currentBank);
}

void PnlHidePanel(PanelType *panel)
{
	if(!panel->visible) return;

	panel->visible = 0;
	WinPushDrawState();
	WinSetBackColor(UIColorGetTableEntryIndex(UIObjectFill));
	WinEraseRectangle(&panel->bounds, 0);
	WinPopDrawState();
}

void PnlShowPanel(PanelType *panel)
{
	if(panel->visible) return;

	panel->visible = 1;
	PnlUpdate(panel, NULL);
	_PnlDrawPanel(panel);
}

void PnlDestroyPanel(PanelType *panel)
{
	PnlHidePanel(panel);
	WinDeleteWindow(panel->_offscreen, false);
}

Boolean	PnlHandleEvent(PanelType *panel, const EventType *e)
{
	PanelEventType newEvent;

	switch(e->eType) {

	case penDownEvent:
		if(panel->visible && RctPtInRectangle(e->screenX, e->screenY, &panel->bounds)) {
			UInt16 x, y, row, column, field, index, sensor;

			x = e->screenX - panel->bounds.topLeft.x;
			y = e->screenY - panel->bounds.topLeft.y;

			// is it an intercell space?
			if(    (y % 10) == 9
			     || x == 79 || x == 80
			     || x == 29 || x == 109) return true;

			row = y / 10;
			column = x / 81;
			field = (x % 81) > 29;
			index = panel->currentBank*8 + row*2 + column;

			sensor = sensorIndex[index];

			if(panel->sensor[sensor].exists ||
				panel->_mode == PANEL_SINGLESELECT_SLUG)
			{
//				UInt16 i;
//				UInt8 oldSelection = sensorSelection;

				UInt32 graph = sensorGraphMask;
				UInt32 capture = sensorCaptureMask;
				UInt8 selection = sensorSelection;

				UInt32 mask = 0;
				UInt8 i;


				if(field == 1) {	//SW
					if(panel->_mode == PANEL_MULTISELECT) {
						for(i = 0; i < 32; i++)
							if(sensorIndex[i] == sensorIndex[index])
								mask |= 1L << i;

						graph = graph & mask ? graph & ~mask : graph | mask;
					} else if(panel->_mode == PANEL_SINGLESELECT_VAL) {
						selection = index;
					}
				} else {
					if(panel->_mode == PANEL_SINGLESELECT_SLUG) {
						selection = index;
					} else {
						mask = diaMod->sensor[sensor].mask;
						capture = capture & mask ? capture & ~mask : capture | mask;
					}
				}

				EvtCopyEvent((EventType*)e, (EventType*)&newEvent);
				newEvent.eType = pnlSelectEvent;
				newEvent.data.pnlSelect.panelID = panel->id;
				newEvent.data.pnlSelect.pPanel = panel;
				newEvent.data.pnlSelect.sensorCaptureMask = capture;
				newEvent.data.pnlSelect.sensorGraphMask = graph;
				newEvent.data.pnlSelect.sensorSelection = selection;
				EvtAddEventToQueue((EventType*)&newEvent);

				SndPlaySystemSound(sndClick);
			}

			return true;
		}
		break;

	default:
		break;
	}
	return false;
}

/*
 *  Set panel mode, either PANEL_SINGLESELECT or PANEL_MULTISELECT
 */

void PnlSetMode(PanelType *panel, panelModeEnum numeric)
{
	panel->_mode = numeric;
//	PnlUpdate(panel, NULL);
	_PnlDrawPanel(panel);
}


//void PnlSetSelection(PanelType *panel, UInt8 selection)
//{
//	sensorSelection = selection;
//	// refresh if in single select mode
//	if(panel->_mode == PANEL_SINGLESELECT_VAL ||
//		panel->_mode == PANEL_SINGLESELECT_SLUG)
//	{
//		_PnlDrawPanel(panel);
//	}
//}
