/*
 *  $Id: panel.h,v 1.8 2003/11/26 15:54:15 kogan Exp $
 *
 *  Copyright (c) 2003, Dmitry Yurtaev <dm1try@umail.ru>
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#ifndef __PANEL_H__
#define __PANEL_H__

#include <PalmTypes.h>
#include <Rect.h>
#include <Event.h>
#include "graph.h"

/*
 *
 */

#define PREFS_UNIT_METRIC	0
#define PREFS_UNIT_ENGLISH	1
#define PREFS_UNIT_NUMERIC	2

typedef enum panelModeEnum {
	PANEL_MULTISELECT,
	PANEL_SINGLESELECT_VAL,
	PANEL_SINGLESELECT_SLUG,
} panelModeEnum;

typedef struct PanelType {
	RectangleType	bounds;
	UInt16			id;
	UInt16			rows;
	UInt8			currentBank;

//	UInt32			capture;
//	UInt32			graph;
//	UInt8			selection;

	DiaSensorType	*sensor;

	WinHandle		_offscreen;
	DiaSampleType	_sample;
	panelModeEnum	_mode;

	UInt16		visible : 1;
} PanelType;

enum panelEventsEnum {
	pnlSelectEvent = firstUserEvent + 20
};


typedef struct PanelEventType {
	eventsEnum	eType;
	Boolean		penDown;
	UInt8		tapCount;
	Int16		screenX;
	Int16		screenY;
	union {
		struct _GenericEventType generic;

		struct pnlSelect {
			UInt16		panelID;
			PanelType	*pPanel;
			UInt32		sensorCaptureMask;
			UInt32		sensorGraphMask;
			UInt8		sensorSelection;
		} pnlSelect;

	} data;
} PanelEventType;


/*
 *
 */

void	PnlCreatePanel(PanelType *panel, UInt16 id,
		DiaSensorType *sensor, UInt8 currentBank,
		Coord x, Coord y, Coord width, Coord height
	);

void	PnlDestroyPanel(PanelType *panel);

void	PnlHidePanel(PanelType *panel);

void	PnlShowPanel(PanelType *panel);

void	PnlUpdate(PanelType *panel, const DiaSampleType* sample);

void	PnlSetBank(PanelType *panel, UInt8 bank);

Boolean	PnlHandleEvent(PanelType *panel, const EventType *event);

void	PnlSetMode(PanelType *panel, panelModeEnum mode);

//void	PnlSetSelection(PanelType *panel, UInt8 selection);

void	PnlRefresh(PanelType *panel);

#endif //__PANEL_H__
