/*
 *  $Id: selector.c,v 1.1 2003/11/26 15:52:43 kogan Exp $
 *
 *  Copyright (c) 2003, Dmitry Yurtaev <dm1try@umail.ru>
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#include <PalmOS.h>
#include <Font.h>
#include "dia.h"
#include "selector.h"
#include "mmcd.h"

#define WIDTH 37

extern Boolean colorMode;

UInt16 sensorUsageMap[16];

#define USAGE_CLEAR		MemSet(sensorUsageMap, 32, 0);
#define USAGE_SET(s)	(sensorUsageMap[(s) >> 4] |= (1 << ((s) & 0x0F)))
#define USAGE_GET(s)	((sensorUsageMap[(s) >> 4] & (1 << ((s) & 0x0F))) != 0)

void _SlDrawButton(SelectorType *selector, UInt16 row, UInt16 column, Boolean blit)
{
	RectangleType rect = { { column * (WIDTH+1), (row - selector->pos) * 10 + 1 }, { WIDTH, 9 } };
	RectangleType fullRect = { { 0, 0 }, selector->bounds.extent };
	WinHandle oldWin;
	FontID oldFontId;
	UInt16 labelWidth, labelLen;
	UInt8 sensor = row * 4 + column;

	// paint to offscreen
	oldWin = WinSetDrawWindow(selector->_offscreen);
	WinPushDrawState();

	// if sensor doesn't exist just erase its button
	if(sensor >= selector->nItems) {
		WinSetBackColor(UIColorGetTableEntryIndex(UIObjectFill));
		WinEraseRectangle(&rect, 0);
	} else {
		Boolean active = sensor == selector->selection;
//		Char label[16];
		const Char *label = diaMod->sensor[sensor].exists ?
			diaMod->sensor[sensor].slug :
			"-";

//		StrPrintF(label, "%u", (UInt16)(column == 0 ? selector->pos : row));


		if(active) {
			WinSetBackColor(UIColorGetTableEntryIndex(UIObjectSelectedFill));
			WinSetForeColor(UIColorGetTableEntryIndex(UIObjectSelectedForeground));
			WinSetTextColor(UIColorGetTableEntryIndex(UIObjectSelectedForeground));
		} else {
			WinSetBackColor(UIColorGetTableEntryIndex(UIObjectFill));
			WinSetForeColor(UIColorGetTableEntryIndex(UIObjectForeground));
			WinSetTextColor(UIColorGetTableEntryIndex(UIObjectForeground));
		}

		WinEraseRectangle(&rect, 0);

		if((labelLen = StrLen(label)))
		{
			oldFontId = FntSetFont(
				(USAGE_GET(sensor) &&
					diaMod->sensor[sensor].exists)
				? boldFont : stdFont);

			labelWidth = FntCharsWidth(label, labelLen) - 2;
	
			WinSetClip(&rect);
			WinDrawChars(label, labelLen,
				rect.topLeft.x + (rect.extent.x - labelWidth) / 2,
				rect.topLeft.y + ((Int16)rect.extent.y - FntCharHeight()) / 2
			);
			WinSetClip(&fullRect);
			FntSetFont(oldFontId);
		}
	}

	WinPopDrawState();
	WinSetDrawWindow(oldWin);

	if(blit && selector->visible)
		WinCopyRectangle(selector->_offscreen, oldWin, &rect, selector->bounds.topLeft.x + rect.topLeft.x, selector->bounds.topLeft.y + rect.topLeft.y, winPaint);
}

void _SlRefreshUsage()
{
	UInt16 i;

	USAGE_CLEAR;
	for(i = 0; i < 32; i++)
		USAGE_SET(sensorIndex[i]);
}

void _SlDrawSelector(SelectorType *selector)
{
	UInt16 row, column;
	RectangleType rect = { { 0, 0 }, selector->bounds.extent };

	_SlRefreshUsage();

	for(row = 0; row < 6; row++)
		for(column = 0; column < 4; column++)
			_SlDrawButton(selector, row + selector->pos, column, false);

	if(selector->visible)
		WinCopyRectangle(selector->_offscreen, WinGetDisplayWindow(), &rect, selector->bounds.topLeft.x, selector->bounds.topLeft.y, winPaint);
}

void SlCreateSelector(SelectorType *selector, UInt8 nItems, UInt8 pos,
	UInt8 selection, Coord x, Coord y, Coord width, Coord height)
{
	Err err;
	WinHandle oldWin;

	selector->selectionInProgress = false;
	selector->nItems = nItems;
	selector->rows = nItems / 4 + (nItems % 4 ? 1 : 0);

	selector->bounds.topLeft.x = x;
	selector->bounds.topLeft.y = y;
	selector->bounds.extent.x = width;
	selector->bounds.extent.y = height;

	selector->visible = 1;

	selector->selection = selection;

	selector->_offscreen = WinCreateOffscreenWindow(width, height, nativeFormat, &err);

	oldWin = WinSetDrawWindow(selector->_offscreen);
	WinPushDrawState();
	WinSetBackColor(UIColorGetTableEntryIndex(UIObjectFill));
	WinEraseWindow();
	WinPopDrawState();
	WinSetDrawWindow(oldWin);

	SlSetPos(selector, pos);
}

void SlHideSelector(SelectorType *selector)
{
	if(!selector->visible) return;

	selector->visible = 0;
	WinPushDrawState();
	WinSetBackColor(UIColorGetTableEntryIndex(UIObjectFill));
	WinEraseRectangle(&selector->bounds, 0);
	WinPopDrawState();
}

void SlShowSelector(SelectorType *selector)
{
	if(selector->visible) return;

	selector->visible = 1;
	_SlDrawSelector(selector);
}

void SlRedraw(SelectorType *selector)
{
	_SlDrawSelector(selector);
}

void SlDestroySelector(SelectorType *selector)
{
	SlHideSelector(selector);
	WinDeleteWindow(selector->_offscreen, false);
}

Boolean	SlHandleEvent(SelectorType *selector, const EventType *e)
{
	SelectorEventType newEvent;

	switch(e->eType) {

	case penDownEvent:
		if(selector->visible && RctPtInRectangle(e->screenX, e->screenY, &selector->bounds)) {
			UInt16 x, y, row, column, sensor;

			x = e->screenX - selector->bounds.topLeft.x;
			y = e->screenY - selector->bounds.topLeft.y;

			// is it an intercell space?
			if((y % 10) == 9 || (x % 38) == 37) return true;

			row = y / 10;
			if(row >= 6) return true;
            row += selector->pos;
			column = x / 38;
			sensor = row * 4 + column;

			if(sensor < selector->nItems) {
				UInt16 old_row = selector->selection / 4;
				UInt16 old_column = selector->selection % 4;

				selector->selection = sensor;

//				_SlRefreshUsage();

				if(old_row >= selector->pos && old_row < selector->pos + 6)
					_SlDrawButton(selector, old_row, old_column, true);

				_SlDrawButton(selector, row, column, true);

				EvtCopyEvent((EventType*)e, (EventType*)&newEvent);
				newEvent.eType = slSelectEvent;
				newEvent.data.slSelect.pSelector = selector;
				newEvent.data.slSelect.selection = selector->selection;
				EvtAddEventToQueue((EventType*)&newEvent);

				SndPlaySystemSound(sndClick);
			
				selector->selectionInProgress = true;
			}

			return true;
		}
		break;

	case penUpEvent:
		if(selector->visible && selector->selectionInProgress) {
			selector->selectionInProgress = false;

			EvtCopyEvent((EventType*)e, (EventType*)&newEvent);
			newEvent.eType = slSelectPenUpEvent;
			newEvent.data.slSelectPenUp.pSelector = selector;
			newEvent.data.slSelectPenUp.selection = selector->selection;
			EvtAddEventToQueue((EventType*)&newEvent);

			return true;
		}
		break;

	default:
		break;
	}
	return false;
}

void SlSetSelection(SelectorType *selector, UInt8 selection)
{
	if(selector->selection == selection) return;
	selector->selection = selection;
	_SlDrawSelector(selector);
}

void SlSetPos(SelectorType *selector, UInt8 pos)
{
	if(selector->pos == pos) return;
	selector->pos = pos;
	_SlDrawSelector(selector);
}

/* Ensure visibility of current selection: modify scroll pos and redraw if nessesary */
void SlEnsureVisible(SelectorType *selector)
{
	UInt8 row = selector->selection / 4;

	if(row <= selector->pos)
		SlSetPos(selector, row > 0 ? row-1 : row);
	else if(row >= selector->pos+6-1)
		SlSetPos(selector, row+1 < selector->rows ? row+2-6 : row+1-6);
}
