/*
 *  $Id: selector.h,v 1.1 2003/11/26 15:52:43 kogan Exp $
 *
 *  Copyright (c) 2003, Dmitry Yurtaev <dm1try@umail.ru>
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#ifndef __SELECTOR_H__
#define __SELECTOR_H__

#include <PalmTypes.h>
#include <Rect.h>
#include <Event.h>
#include "graph.h"

/*
 *
 */

typedef struct SelectorType {
	RectangleType	bounds;			/* selector bounds						*/
	UInt8			selection;		/* sensor index in module sensors array	*/
	WinHandle		_offscreen;		/* drawing buffer						*/
	UInt8			pos;			/* index of first visible row			*/
	UInt8			rows;			/* total number of rows					*/
	UInt8			nItems;			/* total number of items				*/
	UInt16			visible : 1;	/* selector visible						*/
	Boolean			selectionInProgress;
} SelectorType;

enum selectorEventsEnum {
	slSelectEvent		= firstUserEvent + 30,
	slSelectPenUpEvent	= firstUserEvent + 31,
};


typedef struct SelectorEventType {
	eventsEnum	eType;
	Boolean		penDown;
	UInt8		tapCount;
	Int16		screenX;
	Int16		screenY;
	union {
		struct _GenericEventType generic;

		struct slSelect {
			SelectorType	*pSelector;
			UInt8			selection;
		} slSelect;

		struct slSelectPenUp {
			SelectorType	*pSelector;
			UInt8			selection;
		} slSelectPenUp;
	} data;
} SelectorEventType;


/*
 *
 */

void	SlCreateSelector(SelectorType *selector, UInt8 nItems, UInt8 pos,
			UInt8 selection, Coord x, Coord y, Coord width, Coord height);

void	SlDestroySelector(SelectorType *selector);

void	SlHideSelector(SelectorType *selector);

void	SlShowSelector(SelectorType *selector);

void	SlRedraw(SelectorType *selector);

void	SlSetPos(SelectorType *selector, UInt8 pos);

Boolean	SlHandleEvent(SelectorType *selector, const EventType *event);

//void	SlSetMode(SelectorType *selector, selectorModeEnum mode);

void	SlSetSelection(SelectorType *selector, UInt8 selection);

/* Ensure visibility of current selection: modify scroll pos and redraw if nessesary */
void	SlEnsureVisible(SelectorType *selector);

#endif //__PANEL_H__
