/* 
 *  $Id: vb12.c,v 1.3 2003/11/26 15:54:15 kogan Exp $
 *
 *  Copyright (c) 2003, Sergey Kogan
 *
 *  This is free software; you can redistribute it and/or modify it under the
 *  terms of the GNU General Public License as published by the Free Software
 *  Foundation; either version 2, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 */

#include <PalmOS.h>
#include "../config.h"
#include "../dia.h"
#include "vb12.h"
#include "format.h"
#include "../kwp2/kwp2.h"
#include "../kwp2/format.h"


/*** constants ***/

/* sensors */
const DiaSensorType vb12SensorDefault[] = {
//
// E - exists, Sn - sound, P - polarity,
// Thr - threshold, Clr - color
//
//addr  mask        slug    format			scale		compare		E  Sn P Thr color						value        desc
{ 0x00, 0x00000000L, "XXXX", f_DEC,			s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

{ 0x09, 0x00000001L, "RPM",  f_kwp2RPM,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xFF, 0x00, 0x00},	"?\0      ", "Engine speed" },
{ 0x11, 0x00000001L, "IPW",  f_kwp2IPW,		s_kwp2IPW,	c_WORD,		1, 0, 0, 0, {0, 0xFF, 0xA0, 0x00},	"?\0      ", "Injector pulse width" },
{ 0x08, 0x00000001L, "TPS",  f_PERCENT,		s_kwp2TPS,	c_BYTE,		1, 0, 0, 0, {0, 0xFF, 0x00, 0xFF},	"?\0      ", "Throttle position" },
{ 0x07, 0x00000001L, "AFR",  f_kwp2AFR,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xD0, 0x90, 0x00},	"?\0      ", "Air/Fuel ratio" },
{ 0x0E, 0x00000001L, "VS",   f_kwp2VS,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0x00, 0xA0},	"?\0      ", "Vehicle speed" },
{ 0x13, 0x00000001L, "MAF",  f_kwp2MAF,		s_kwp2MAF,	c_WORD,		1, 0, 0, 0, {0, 0x00, 0x00, 0xFF},	"?\0      ", "Mass air flow" },
{ 0x0D, 0x00000001L, "SA",   f_kwp2SA,		s_kwp2SA,	c_BYTE,		1, 0, 0, 0, {0, 0x60, 0x60, 0x60},	"?\0      ", "Spark advance" },
{ 0x15, 0x00000001L, "CAF",  f_kwp2CAF,		s_kwp2CAF,	c_WORD,		1, 0, 0, 0, {0, 0xA0, 0xA0, 0xFF},	"?\0      ", "Cycle air flow" },

{ 0x06, 0x00000001L, "ECT",  f_kwp2ECT,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0xFF, 0x00},	"?\0      ", "Engine coolant temperature" },
{ 0x17, 0x00000001L, "HFUE", f_kwp2HFUE,	s_kwp2HFUE,	c_WORD,		1, 0, 0, 0, {0, 0xFF, 0xA0, 0xA0},	"?\0      ", "Hour fuel consumption" },
{ 0x0F, 0x00000001L, "BATT", f_kwp2BATT,	s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xA0, 0xFF, 0xA0},	"?\0      ", "Battery voltage" },
{ 0x19, 0x00000001L, "FUEL", f_kwp2FUEL,	s_kwp2FUEL,	c_WORD,		1, 0, 0, 0, {0, 0xA0, 0x60, 0x00},	"?\0      ", "Fuel consumption" },
{ 0x0B, 0x00000001L, "ISCP", f_DEC,			s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0xFF, 0xFF},	"?\0      ", "Idle speed control position" },
{ 0x0A, 0x00000001L, "XRPM", f_kwp2XRPM,	s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xA0, 0x00, 0xA0},	"?\0      ", "Idle engine speed" },
{ 0x0C, 0x00000001L, "IPC",  f_kwp2IPC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xA0, 0xA0, 0x00},	"?\0      ", "Injector pulse correction" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,			s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

{ 0x1e, 0x00000002L, "KNKv", f_kwp2ADC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x30, 0x30, 0x30},	"?\0      ", "Knock sensor voltage" },
{ 0x1f, 0x00000002L, "ECTv", f_kwp2ADC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0xA0, 0x00},	"?\0      ", "ECT voltage" },
{ 0x20, 0x00000002L, "MAFv", f_kwp2ADC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x00, 0x00, 0xA0},	"?\0      ", "Mass air flow sensor voltage" },
{ 0x21, 0x00000002L, "BATv", f_kwp2BATv,	s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x60, 0xA0, 0x60},	"?\0      ", "Battery voltage" },
{ 0x22, 0x00000002L, "O2Sv", f_kwp2ADC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0x60, 0x60, 0x60},	"?\0      ", "O2 sensor voltage" },
{ 0x23, 0x00000002L, "TPSv", f_kwp2ADC,		s_DECBYTE,	c_BYTE,		1, 0, 0, 0, {0, 0xA0, 0x00, 0xA0},	"?\0      ", "Throttle position sensor voltage" },
//{ 0x04, 0x00000001L, "XXXX", f_kwp2KNCK,	s_kwp2KNCK,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,			s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,			s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,			s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },

{ 0x00, 0x00000001L, "XXXX", f_DEC,			s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,			s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0xD0, 0xD0, 0xD0},	"?\0      ", "CO correction" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,			s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,			s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,			s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,			s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,			s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
{ 0x00, 0x00000001L, "XXXX", f_DEC,			s_DECBYTE,	c_BYTE,		0, 0, 0, 0, {0, 0x00, 0x00, 0x00},	"?\0      ", "" },
};

const DiaTimingType	vb12TimingDefault = {
	0,			/* Min P1 */
	20,			/* Max P1 */
	25,			/* Min P2 */
	60,			/* Max P2 */
	110,		/* Min P3 */
	5000,		/* Max P3 */
	0,			/* Min P4 */
	20			/* Max P4 */
};
DiaTimingType		vb12Timing;
